from yixue import Configs
from yixue.Models import BaZi, GeJuItem, SiZhuGeJu, SiZhu
from yixue.geju import CalcXiongJi


# 要注意，这里其实有2个概念，一个叫天干地支影响，一个叫格局。
# 天干地支影响指的是当10天干12地支到来时，对原局、大运、流年等影响，可以用来看哪些年份、月份的利弊。
# 客户端页面上的天干地支选择指的就是这个，也可以用来看五行应该怎么增补最有利
# 格局指的是八字的日元影响下，高能关系转换为十神会是什么，这里和原局、大运这些没有关系，只是一种由日元
# 决定下的八字的适配性。比如乙木日元，一旦遇到辛，必然七杀攻身。乙木日元，比如酉辰合，促成了七杀合正财，这里跟
# 原局、大运等是否有辰和酉这两个因素没有关系。

def calc_tian_gan(bazi: BaZi):
    bazi.tiangan_geju = {}
    lushen = bazi.shishen_guanxi["禄神"]
    banlu = bazi.shishen_guanxi["半禄"]
    __calc_geju__(bazi.ri_gan.text, bazi.tiangan_geju, bazi.all_tian_gans(), Configs.config_tiangan_list, lushen, banlu,
                  0, True)

    bazi.dayun_tiangan_geju = {}
    dayun_tgs = []
    for dayun in bazi.dayun:
        if dayun.is_dayun:
            dayun_tgs.append(dayun.gan_zhu)
    __calc_geju__(bazi.ri_gan.text, bazi.dayun_tiangan_geju, dayun_tgs, Configs.config_tiangan_list, lushen, banlu, 1,
                  True)

    bazi.liunian_tiangan_geju = {}
    liunian_tgs = []
    for dayun in bazi.dayun:
        if not dayun.is_dayun:
            continue
        for ln in dayun.liunian:
            liunian_tgs.append(ln.gan_zhu)
    __calc_geju__(bazi.ri_gan.text, bazi.liunian_tiangan_geju, liunian_tgs, Configs.config_tiangan_list, lushen, banlu,
                  2, True)


def calc_di_zhi(bazi: BaZi):
    bazi.dizhi_geju = {}
    lushen = bazi.shishen_guanxi["禄神"]
    banlu = bazi.shishen_guanxi["半禄"]
    __calc_geju__(bazi.ri_gan.text, bazi.dizhi_geju, bazi.all_di_zhis(), Configs.config_dizhi_list, lushen, banlu, 0,
                  False)

    bazi.dayun_dizhi_geju = {}
    dayun_dzs = []
    for dayun in bazi.dayun:
        if dayun.is_dayun:
            dayun_dzs.append(dayun.zhi_zhu)
    __calc_geju__(bazi.ri_gan.text, bazi.dayun_dizhi_geju, dayun_dzs, Configs.config_dizhi_list, lushen, banlu, 1,
                  False)

    bazi.liunian_dizhi_geju = {}
    liunian_dzs = []
    for dayun in bazi.dayun:
        if not dayun.is_dayun:
            continue
        for ln in dayun.liunian:
            liunian_dzs.append(ln.zhi_zhu)
    __calc_geju__(bazi.ri_gan.text, bazi.liunian_dizhi_geju, liunian_dzs, Configs.config_dizhi_list, lushen, banlu,
                  2, False)


def count_geju_score(bazi: BaZi):
    bazi.geju_score = {}
    effects = {}
    for wx in Configs.config_wuxing_list:
        bazi.geju_score[wx] = 0
        effects[wx] = 0
    for ss in Configs.config_shishen_list:
        bazi.geju_score[ss] = 0
        effects[ss] = 0

    lst = [bazi.tiangan_geju, bazi.dizhi_geju]
    for item in lst:
        for key in item:
            geju = item[key]
            if geju.total == 0:
                continue
            shishen = Configs.config_shishen[bazi.ri_gan.text][key]
            wuxing = Configs.config_wuxing[key]
            effects[wuxing] += 1
            effects[shishen] += 1
            bazi.geju_score[shishen] += geju.total
            bazi.geju_score[wuxing] += geju.total

    # print(effects)
    # 把总分除以个数，计算平均分
    for key in bazi.geju_score:
        value = bazi.geju_score[key]
        amount = effects[key]
        if value == 0 or amount == 0:
            continue
        avg = round(value / amount, 1)
        bazi.geju_score[key] = avg


def count_dayun_geju_score(bazi: BaZi):
    # key 大运
    # value :{
    #   "官": 0,
    #   "火" :10,
    # }

    bazi.dayun_geju_score = {}
    effects = {}

    # 根据大运做初始化
    for dayun in bazi.dayun:
        if not dayun.is_dayun:
            continue

        temp = {}
        dayun_key = dayun.gan_text + dayun.zhi_text
        bazi.dayun_geju_score[dayun_key] = temp
        effects[dayun_key] = {}

        for wx in Configs.config_wuxing_list:
            temp[wx] = 0
            effects[dayun_key][wx] = 0

        for ss in Configs.config_shishen_list:
            temp[ss] = 0
            effects[dayun_key][ss] = 0

    # 计算并填装数据
    for target in [bazi.dayun_tiangan_geju, bazi.dayun_dizhi_geju]:
        for gan_zhi in target:  # key = 干支
            shishen = Configs.config_shishen[bazi.ri_gan.text][gan_zhi]
            wuxing = Configs.config_wuxing[gan_zhi]
            sz_ge_ju = target[gan_zhi]  # SiZhuGeJu
            temp_values = {}
            for item in sz_ge_ju.items:  # GeJuItem
                total = item.calc_total()
                if total == 0:
                    continue

                obj = bazi.dayun_geju_score[item.trigger]
                obj[shishen] += total
                obj[wuxing] += total

                if not temp_values.__contains__(item.trigger):
                    temp_values[item.trigger] = {}
                    __init_obj_of_wuxing_shishen__(temp_values[item.trigger])
                temp_values[item.trigger][wuxing] += total
                temp_values[item.trigger][shishen] += total

            # if temp_values.__contains__("丙寅"):
            # print(temp_values["丙寅"])
            for dy in temp_values:
                dt = temp_values[dy]
                for wuxing in Configs.config_wuxing_list:
                    if dt[wuxing] != 0:
                        effects[dy][wuxing] += 1
                for shishen in Configs.config_shishen_list:
                    if dt[shishen] != 0:
                        effects[dy][shishen] += 1

    # print(11, bazi.dayun_geju_score["丙寅"])
    # print(12, effects["丙寅"])

    # 把总分除以个数，计算平均分
    for dayun in bazi.dayun_geju_score:
        obj = bazi.dayun_geju_score[dayun]
        eft = effects[dayun]
        for key in obj:
            value = obj[key]
            amount = eft[key]
            if value == 0 or amount == 0:
                continue

            avg = round(value / amount, 1)
            obj[key] = avg
            # print(value, amount, avg, dayun, key)


def count_liunian_geju_score(bazi: BaZi):
    bazi.liunian_geju_score = {}
    effects = {}
    # 根据流年做初始化
    for dayun in bazi.dayun:
        if not dayun.is_dayun:
            continue
        for liunian in dayun.liunian:
            temp = {}
            liunian_key = dayun.gan_text + dayun.zhi_text + liunian.gan_text + liunian.zhi_text
            bazi.liunian_geju_score[liunian_key] = temp
            effects[liunian_key] = {}
            for wx in Configs.config_wuxing_list:
                temp[wx] = 0
                effects[liunian_key][wx] = 0
            for ss in Configs.config_shishen_list:
                temp[ss] = 0
                effects[liunian_key][ss] = 0

    # 计算并填装数据
    for target in [bazi.liunian_tiangan_geju, bazi.liunian_dizhi_geju]:
        for gan_zhi in target:  # key = 干支
            shishen = Configs.config_shishen[bazi.ri_gan.text][gan_zhi]
            wuxing = Configs.config_wuxing[gan_zhi]
            sz_ge_ju = target[gan_zhi]  # SiZhuGeJu
            temp_values = {}
            for item in sz_ge_ju.items:  # GeJuItem
                total = item.calc_total()
                if total == 0:
                    continue

                obj = bazi.liunian_geju_score[item.trigger]
                obj[shishen] += total
                obj[wuxing] += total

                if not temp_values.__contains__(item.trigger):
                    temp_values[item.trigger] = {}
                    __init_obj_of_wuxing_shishen__(temp_values[item.trigger])
                temp_values[item.trigger][wuxing] += total
                temp_values[item.trigger][shishen] += total

            for ln_key in temp_values:
                dt = temp_values[ln_key]
                for wuxing in Configs.config_wuxing_list:
                    if dt[wuxing] != 0:
                        effects[ln_key][wuxing] += 1
                for shishen in Configs.config_shishen_list:
                    if dt[shishen] != 0:
                        effects[ln_key][shishen] += 1

    for key in bazi.liunian_geju_score:
        obj = bazi.liunian_geju_score[key]
        eft = effects[key]
        for atr in obj:
            value = obj[atr]
            amount = eft[atr]
            if value == 0 or amount == 0:
                continue
            avg = round(value / amount, 1)
            obj[atr] = avg


def __init_obj_of_wuxing_shishen__(obj: {}):
    for wx in Configs.config_wuxing_list:
        obj[wx] = 0

    for ss in Configs.config_shishen_list:
        obj[ss] = 0


def __calc_dayun_geju_score__(bazi: BaZi):
    effects = {}
    extra_key = "运"
    for wx in Configs.config_wuxing_list:
        bazi.geju_score[wx + extra_key] = 0
        effects[wx + extra_key] = 0
    for ss in Configs.config_shishen_list:
        bazi.geju_score[ss + extra_key] = 0
        effects[ss + extra_key] = 0

    lst = [bazi.tiangan_geju, bazi.dizhi_geju]
    for item in lst:
        for key in item:
            geju = item[key]
            # shishen = Configs.config_shishen[bazi.ri_gan.text][key]
            # wuxing = Configs.config_wuxing[key]
            # effects[wuxing] += 1
            # effects[shishen] += 1
            # bazi.geju_score[shishen + extra_key] += geju.total
            # bazi.geju_score[wuxing + extra_key] += geju.total
            for temp_key in geju.dayun_scores:
                shishen = Configs.config_shishen[bazi.ri_gan.text][temp_key]
                wuxing = Configs.config_wuxing[temp_key]
                effects[wuxing + extra_key] += 1
                effects[shishen + extra_key] += 1
                bazi.geju_score[shishen + extra_key] += geju.total
                bazi.geju_score[wuxing + extra_key] += geju.total

    # 把总分除以个数，计算平均分
    for key in bazi.geju_score:
        if not "运" in key:
            continue
        value = bazi.geju_score[key]
        amount = effects[key]
        avg = round(value / amount, 1)
        bazi.geju_score[key] = avg


def check_xiong_ji(bazi: BaZi):
    CalcXiongJi.run(bazi)


def __check_liuhehuasha__(geju: SiZhuGeJu, lushen: str, banlu: [str]):
    # 检查四柱格局中是否有地支六合化煞
    hasHe = False
    hasSha = False
    heList = []
    for item in geju.items:
        if item.guanxi in ["合", "暗合"]:
            hasHe = True
            heList.append(item)
        elif item.guanxi in ["冲", "刑", "穿", "破"]:
            hasSha = True

    if hasHe and hasSha:
        for item in heList:
            item.clear_score()
            __calc_score__(item, lushen, banlu, 30)


def __calc_geju__(riyuan: str, result, szAry: [SiZhu], texts: [str], lushen: str, banlu: [str], trigger_mode: int,
                  is_tiangan: bool):
    for text in texts:
        szGeJu = SiZhuGeJu()
        szGeJu.yudao = text
        for sz in szAry:
            if sz.text == "无":
                continue
            key = text + sz.text
            if Configs.config_gaoneng_query.__contains__(key):
                gns = Configs.config_gaoneng_query[key]
                for gn in gns:
                    item = GeJuItem()
                    item.arg1 = text
                    item.arg2 = sz.text
                    if trigger_mode == 1:
                        item.trigger = sz.dayun_text
                    elif trigger_mode == 2:
                        item.trigger = sz.dayun_text + sz.liunian_text
                    item.shishen1 = Configs.config_shishen[riyuan][text]
                    item.shishen2 = Configs.config_shishen[riyuan][sz.text]
                    item.guanxi = gn["guanxi"]
                    __calc_score__(item, lushen, banlu, 0)
                    szGeJu.items.append(item)
        if not is_tiangan:
            __check_liuhehuasha__(szGeJu, lushen, banlu)
        szGeJu.count_score()
        result[text] = szGeJu


def __calc_score__(item: GeJuItem, lushen: str, banlu: [str], extra: int):
    # 评分由两个部分组成，一个是关系分，一个是排序分
    # 关系分指的是高能关系的评分，排序分指的是当发生某些关系时，受到好坏程度的影响高低
    # 最终分数等于 关系分(十位) + 高能分（个位）
    # 另外，如果对禄神起作用，将产生额外的禄神系数，禄神系数是一个倍数，作用在排序分上，用来增加或减少得到的排序分
    # 最终分数=关系分+排序分*禄神系数
    # 天干的合有可能虚透，分数要少一些(50)，地支的合如果同柱内不存在其他负面关系，分数也会少一些(50),暗合也是这个规则

    order_value = __get__config_score__(item)
    # 默认0分，表示不发生高能关系时，不好也不坏
    guanxi_value = Configs.get_value(Configs.config_gaoneng_guanxi_score, item.guanxi)
    if guanxi_value is None:
        guanxi_value = 0
    guanxi_value += extra

    lushen_param = 0
    if item.arg1 == lushen or item.arg2 == lushen:
        lushen_param = 1
    elif item.arg1 in banlu or item.arg2 in banlu:
        lushen_param = 0.5

    if lushen_param > 0:
        lushen_score = 0
        if Configs.config_gaoneng_lushen_score.__contains__(item.guanxi):
            lushen_score = Configs.config_gaoneng_lushen_score[item.guanxi]
        new_order_value = lushen_score * lushen_param / 100
        new_order_value = (1 + new_order_value) * order_value
        order_value = round(new_order_value, 0)

    if item.shishen2 in ["官", "杀"]:
        item.guan_sha += order_value + guanxi_value
    elif item.shishen2 in ["比", "劫"]:
        item.bi_jie += order_value + guanxi_value
    elif item.shishen2 in ["财", "才"]:
        item.cai_xing += order_value + guanxi_value
    elif item.shishen2 in ["印", "枭"]:
        item.yin_shou += order_value + guanxi_value
    elif item.shishen2 in ["食", "伤"]:
        item.shi_shang += order_value + guanxi_value


def __get__config_score__(item: GeJuItem):
    score = 0
    if item.guanxi != "自刑":
        try:
            score = Configs.config_gaoneng_score[item.guanxi][item.shishen1][item.shishen2]
        except Exception:
            print("not found gaoneng score:", item.guanxi, item.shishen1, item.shishen2)
    return score
