import datetime

from yixue import Configs


class BaseYiXue(object):
    def __repr__(self):
        result = self.__class__.__name__ + "("
        for attr in self.__dict__:
            if attr.startswith("_") or attr.startswith("__") or attr == "__tablename__":
                continue
            result += attr + "=" + str(getattr(self, attr)) + " "
        result += ")"
        return result

    def __resolve_base_type__(self, obj, attr, value):
        obj[attr] = value

    def __resolve_yixue_object(self, obj, attr, value):
        obj[attr] = value.to_response()

    def __resolve__(self, value):
        if value is None:
            return None
        tp = type(value)
        if tp in [str, int, float, bool]:
            return value
        elif tp is datetime.datetime:
            return str(value)
        elif issubclass(tp, BaseYiXue):
            return value.to_response()
        elif tp is list:
            temp = []
            for list_item in value:
                temp.append(self.__resolve__(list_item))
            return temp
        elif tp is dict:
            temp = {}
            for k, v in value.items():
                temp[k] = self.__resolve__(v)
            return temp
        else:
            print("无法处理的类型：", tp, value)
            return None

    def to_response(self):
        result = {}
        for attr in self.__dict__:
            if attr.startswith("_") or attr.startswith("__") or attr == "__tablename__":
                continue
            value = getattr(self, attr)
            result[attr] = self.__resolve__(value)
        return result


class BaseInfo(BaseYiXue):
    date_mode: int
    year: int
    month: int
    day: int
    hour_mode: int
    hour: int
    minute: int
    area_mode: int
    sheng: str
    shi: str
    qu: str
    nongli_year: int
    nongli_month: str
    nongli_day: str
    nian_gan: str
    nian_zhi: str
    yue_gan: str
    yue_zhi: str
    ri_gan: str
    ri_zhi: str
    shi_gan: str
    shi_zhi: str


class SiZhuResult(BaseYiXue):
    nian_gan: str = None
    nian_zhi: str = None
    yue_gan: str = None
    yue_zhi: str = None
    ri_gan: str = None
    ri_zhi: str = None
    shi_gan: str = None
    shi_zhi: str = None
    sheng: str = None
    shi: str = None
    qu: str = None
    tai_yang_shi: datetime = None


class SiZhuCheckResult(BaseYiXue):
    error: bool
    adv_nian: [str]
    adv_yue: [str]
    adv_ri: [str]
    adv_shi: [str]
    adv_date: [str]

    def __init__(self):
        self.error = False
        self.adv_nian = []
        self.adv_yue = []
        self.adv_ri = []
        self.adv_shi = []
        self.adv_date = []


class PowerCounter(BaseYiXue):
    power_raw: {}
    power_muku: {}
    power_gaoneng: {}
    power_zihe: {}

    @classmethod
    def new_counter(cls):
        temp = {}
        for wx in Configs.config_wuxing_list:
            temp[wx] = 0
        for ss in Configs.config_shishen_list:
            temp[ss] = 0
        return temp


class GaoNeng(BaseYiXue):
    arg1: str
    arg2: str
    arg3: str
    wei_zhi1: int
    wei_zhi2: int
    wei_zhi3: int
    wu_xing1: str
    wu_xing2: str
    wu_xing3: str
    shi_shen1: str
    shi_shen2: str
    shi_shen3: str
    text: str
    guanxi: str
    leibie: str
    hehua: str
    zengqiang_shishen: str
    fangxiang: str
    tip: str
    sign: str
    xing_chu: str
    xing_chong: str

    def __init__(self):
        self.arg3 = None
        self.wei_zhi3 = None


class SiZhu(BaseYiXue):
    text: str
    wei_zhi: int
    wu_xing: str
    shi_shen: str
    yin_yang: str
    cang_gan1: str
    cang_gan2: str
    cang_gan3: str
    gan1_wu_xing: str
    gan2_wu_xing: str
    gan3_wu_xing: str
    gan1_shi_shen: str = None
    gan2_shi_shen: str = None
    gan3_shi_shen: str = None
    zhang_sheng_zhu: str
    zhang_sheng_ri: str
    gaonengs: [GaoNeng]
    shenshas: [str]
    is_muku: bool
    is_muku_open: bool
    trigger_gaonengs: {}
    dayun_text: str
    liunian_text: str
    liuyue_text: str
    liuri_text: str

    def __init__(self, text: str, wz: int):
        self.text = text
        self.wei_zhi = wz
        self.gaonengs = []
        self.shenshas = []
        self.trigger_gaonengs = {}
        self.is_muku = Configs.config_muku_list.__contains__(text)
        self.is_muku_open = False

    def has_guanxi_in_gaonengs(self, guanxis: [str]):
        result = False
        for gn in self.gaonengs:
            if gn.guanxi in guanxis:
                result = True
                break
        return result

    def has_shen_sha(self, name: str):
        result = False
        for sh in self.shenshas:
            if sh == name:
                result = True
                break
        return result


class GeJuItem(BaseYiXue):
    arg1: str
    arg2: str
    shishen1: str
    shishen2: str
    guanxi: str
    guan_sha: int = 0
    cai_xing: int = 0
    yin_shou: int = 0
    shi_shang: int = 0
    bi_jie: int = 0
    trigger: str

    def clear_score(self):
        self.guan_sha = 0
        self.cai_xing = 0
        self.yin_shou = 0
        self.shi_shang = 0
        self.bi_jie = 0

    def calc_total(self):
        return self.guan_sha + self.cai_xing + self.yin_shou + self.shi_shang + self.bi_jie


class SiZhuGeJu(BaseYiXue):
    yudao: str
    items: [GeJuItem]
    guan_sha: int = 0
    cai_xing: int = 0
    yin_shou: int = 0
    shi_shang: int = 0
    bi_jie: int = 0
    total: int = 0

    def __init__(self):
        self.items = []

    def count_score(self):
        for item in self.items:
            self.guan_sha += item.guan_sha
            self.cai_xing += item.cai_xing
            self.yin_shou += item.yin_shou
            self.shi_shang += item.shi_shang
            self.bi_jie += item.bi_jie
        self.total = self.guan_sha + self.cai_xing + self.yin_shou + self.shi_shang + self.bi_jie


class XiongJi(BaseYiXue):
    name: str
    guanxi: str
    arg1: str
    arg2: str
    tip: str
    found: bool


class LiuRi(BaseYiXue):
    day: int
    nong_li: str
    xing_qi: str
    gan_text: str
    zhi_text: str
    gan_zhu: SiZhu
    zhi_zhu: SiZhu
    trigger: str


class LiuYue(BaseYiXue):
    month: int
    jie_qi: str
    jie_qi_date: str
    gan_text: str
    zhi_text: str
    gan_zhu: SiZhu
    zhi_zhu: SiZhu
    liuri: [LiuRi]
    trigger: str
    yinyuan_level: int = 0
    yinyuan_tip: str


class LiuNian(BaseYiXue):
    year: int
    gan_text: str
    zhi_text: str
    from_dayun: bool
    age: int
    gan_zhu: SiZhu
    zhi_zhu: SiZhu
    liuyue: []
    trigger: str
    yinyuan_level: int = 0
    yinyuan_tip: str


class DaYun(BaseYiXue):
    gan_text: str
    zhi_text: str
    is_dayun: bool
    start_age: int
    end_age: int
    start_year: int
    end_year: int
    gan_zhu: SiZhu
    zhi_zhu: SiZhu
    liunian: []
    trigger: str


class ZhenDuanItem(BaseYiXue):
    key: str
    content: str
    order: int
    selected: bool
    defaultSelected: bool
    tip: str

    def __init__(self, order: int, *args: str):
        self.order = order
        self.content = ""
        self.selected = False
        self.defaultSelected = False
        self.key = None
        for arg in args:
            self.content += arg

    def set_selected(self, v: bool):
        self.selected = v
        self.defaultSelected = v


class ZhenDuan(BaseYiXue):
    jianyi: [ZhenDuanItem]
    cankao: [ZhenDuanItem]

    def __init__(self):
        self.jianyi = []
        self.cankao = []


class ZhenDuanSet(BaseYiXue):
    wuxing: ZhenDuan
    shishen: ZhenDuan
    zuozhi: ZhenDuan
    menhu: ZhenDuan
    shensha: ZhenDuan
    geju: ZhenDuan
    gaoneng: {}
    jiankang: ZhenDuan
    qinggan: ZhenDuan
    dayun: ZhenDuan
    yinyuan: ZhenDuan

    def __init__(self):
        self.wuxing = ZhenDuan()
        self.shishen = ZhenDuan()
        self.zuozhi = ZhenDuan()
        self.menhu = ZhenDuan()
        self.shensha = ZhenDuan()
        self.geju = ZhenDuan()
        self.gaoneng = {}
        self.jiankang = ZhenDuan()
        self.qinggan = ZhenDuan()
        self.dayun = ZhenDuan()
        self.yinyuan = ZhenDuan()


class PowerQiangRuo(BaseYiXue):
    mu_ruo: bool = False
    huo_ruo: bool = False
    tu_ruo: bool = False
    jin_ruo: bool = False
    shui_ruo: bool = False
    mu_wang: bool = False
    huo_wang: bool = False
    tu_wang: bool = False
    jin_wang: bool = False
    shui_wang: bool = False
    guan_sha_ruo: bool = False
    cai_xing_ruo: bool = False
    yin_shou_ruo: bool = False
    shi_shang_ruo: bool = False
    bi_jie_ruo: bool = False
    guan_sha_wang: bool = False
    cai_xing_wang: bool = False
    yin_shou_wang: bool = False
    shi_shang_wang: bool = False
    bi_jie_wang: bool = False
    guan_sha_guo_wang: bool = False
    cai_xing_guo_wang: bool = False


class TaoHuaZhu(BaseYiXue):
    gan_zhu: SiZhu
    zhi_zhu: SiZhu
    level: int  # 数值越小优先级越高
    tg_keys: [str]
    dz_keys: [str]

    def __init__(self, gan: SiZhu, zhi: SiZhu, level: int):
        self.level = level
        self.gan_zhu = gan
        self.zhi_zhu = zhi
        self.tg_keys = []
        self.dz_keys = []


class QuShiItem(BaseYiXue):
    key: str
    score: int
    reasons: [str]


class LiuNianQuShi(BaseYiXue):
    year: int
    gan_text: str
    zhi_text: str
    age: int
    items: {}


class DaYunQuShi(BaseYiXue):
    is_dayun: bool
    gan_text: str
    zhi_text: str
    start_year: int
    end_year: int
    start_age: int
    end_age: int
    items: {}


class QuShiSet(BaseYiXue):
    dayun: [DaYunQuShi]
    liunian: [LiuNianQuShi]


class MBTIItem(BaseYiXue):
    key: str
    desc: str
    state: int
    shishen: str
    wuxing: str
    power: int
    score: int


class MBTISet(BaseYiXue):
    key: str
    items: [MBTIItem]
    total_score: int
    score_detail: {}
    percentage: int


class BaZi(BaseYiXue):
    is_man: bool
    tai_yang_shi: datetime
    base_info: BaseInfo
    nian_gan: SiZhu
    nian_zhi: SiZhu
    yue_gan: SiZhu
    yue_zhi: SiZhu
    ri_gan: SiZhu
    ri_zhi: SiZhu
    shi_gan: SiZhu
    shi_zhi: SiZhu
    counter: PowerCounter
    power_steps: {}
    shishen_guanxi: {}
    tong_dang: float
    yi_dang: float
    tiangan_geju: {}
    dizhi_geju: {}
    geju_score: {}
    geju_xiongji: {}
    dayun: [DaYun]
    dayun_tiangan_geju: {}
    dayun_dizhi_geju: {}
    dayun_geju_score: {}
    liunian_tiangan_geju: {}
    liunian_dizhi_geju: {}
    liunian_geju_score: {}
    zhenduan: ZhenDuanSet
    qushi: QuShiSet
    mbti: {}

    @staticmethod
    def build(sizhu: SiZhuResult):
        """
        传入参数的顺序必须是：年干、年支、月干、月支、日干、日支、时干、时支
        """
        bazi = BaZi()
        bazi.nian_gan = SiZhu(sizhu.nian_gan, Configs.const_nian_gan)
        bazi.nian_zhi = SiZhu(sizhu.nian_zhi, Configs.const_nian_zhi)
        bazi.yue_gan = SiZhu(sizhu.yue_gan, Configs.const_yue_gan)
        bazi.yue_zhi = SiZhu(sizhu.yue_zhi, Configs.const_yue_zhi)
        bazi.ri_gan = SiZhu(sizhu.ri_gan, Configs.const_ri_gan)
        bazi.ri_zhi = SiZhu(sizhu.ri_zhi, Configs.const_ri_zhi)
        bazi.shi_gan = SiZhu(sizhu.shi_gan, Configs.const_shi_gan)
        bazi.shi_zhi = SiZhu(sizhu.shi_zhi, Configs.const_shi_zhi)
        bazi.tai_yang_shi = sizhu.tai_yang_shi

        bazi.zhenduan = ZhenDuanSet()
        return bazi

    def all_zhus(self):
        """
        返回顺序：年干、年支、月干、月支、日干、日支、时干、时支
        """
        return [self.nian_gan, self.nian_zhi, self.yue_gan, self.yue_zhi, self.ri_gan, self.ri_zhi, self.shi_gan,
                self.shi_zhi]

    def all_tian_gans(self):
        return [self.nian_gan, self.yue_gan, self.ri_gan, self.shi_gan]

    def all_di_zhis(self):
        return [self.nian_zhi, self.yue_zhi, self.ri_zhi, self.shi_zhi]
