from yixue import Configs
from yixue.Models import BaZi, QuShiSet, DaYunQuShi, QuShiItem, SiZhu, GaoNeng, LiuNian, LiuNianQuShi


def calc_qushi(bazi: BaZi):
    bazi.qushi = QuShiSet()
    __dayun_qushi__(bazi)


def __dayun_qushi__(bazi: BaZi):
    bazi.qushi.dayun = []
    bazi.qushi.liunian = []
    lushen = bazi.shishen_guanxi["禄神"]
    banlu = bazi.shishen_guanxi["半禄"]

    for dayun in bazi.dayun:
        if not dayun.is_dayun:
            continue
        qs = DaYunQuShi()
        qs.is_dayun = dayun.is_dayun
        qs.gan_text = dayun.gan_text
        qs.zhi_text = dayun.zhi_text
        qs.start_year = dayun.start_year
        qs.end_year = dayun.end_year
        qs.start_age = dayun.start_age
        qs.end_age = dayun.end_age
        qs.items = {}
        __create_qushi_items__(qs.items)
        __resolve_zhu__(dayun.gan_zhu, qs.items, lushen, banlu, "大运")
        __resolve_zhu__(dayun.zhi_zhu, qs.items, lushen, banlu, "大运")
        for liunian in dayun.liunian:
            __resolve_zhu__(liunian.gan_zhu, qs.items, lushen, banlu, str(liunian.year) + "年")
            __resolve_zhu__(liunian.zhi_zhu, qs.items, lushen, banlu, str(liunian.year) + "年")
            __build_liunian_qu_shi__(liunian, lushen, banlu, bazi)

        bazi.qushi.dayun.append(qs)


def __build_liunian_qu_shi__(liunian: LiuNian, lushen: str, banlu: [str], bazi: BaZi):
    qs = LiuNianQuShi()
    qs.year = liunian.year
    qs.age = liunian.age
    qs.gan_text = liunian.gan_text
    qs.zhi_text = liunian.zhi_text
    qs.items = {}
    __create_qushi_items__(qs.items)
    __resolve_zhu__(liunian.gan_zhu, qs.items, lushen, banlu, str(liunian.year) + "年")
    __resolve_zhu__(liunian.zhi_zhu, qs.items, lushen, banlu, str(liunian.year) + "年")
    for liuyue in liunian.liuyue:
        __resolve_zhu1__(liuyue.gan_zhu, qs.items, lushen, banlu, str(liunian.year) + "年" + str(liuyue.month) + "月")
        __resolve_zhu1__(liuyue.zhi_zhu, qs.items, lushen, banlu, str(liunian.year) + "年" + str(liuyue.month) + "月")
    bazi.qushi.liunian.append(qs)


def __resolve_zhu1__(zhu: SiZhu, items: {}, lushen: str, banlu: [str], title: str):
    ss = zhu.shi_shen
    item = items[ss]
    extra = 0
    if __has_hua_sha__(zhu):
        extra = 30
    for gn in zhu.gaonengs:
        # 排序分，就是合什么，冲什么的不同分数
        order_score = __get_order_score__(gn)
        guanxi_score = __get_guanxi_score__(gn)
        if gn.guanxi in ["合", "暗合"]:
            guanxi_score += extra
        total_score = __calc_total_score__(order_score, guanxi_score, lushen, banlu, gn)
        item.score += total_score
        reason = title + " : " + gn.shi_shen1 + gn.guanxi + gn.shi_shen2 + " => " + str(total_score)
        item.reasons.append(reason)


def __resolve_zhu__(zhu: SiZhu, items: {}, lushen: str, banlu: [str], title: str):
    ss = zhu.shi_shen
    item = items[ss]
    extra = 0
    if __has_hua_sha__(zhu):
        extra = 30
    for gn in zhu.gaonengs:
        # 排序分，就是合什么，冲什么的不同分数
        order_score = __get_order_score__(gn)
        guanxi_score = __get_guanxi_score__(gn)
        if gn.guanxi in ["合", "暗合"]:
            guanxi_score += extra
        total_score = __calc_total_score__(order_score, guanxi_score, lushen, banlu, gn)
        item.score += total_score
        reason = title + " : " + gn.shi_shen1 + gn.guanxi + gn.shi_shen2 + " => " + str(total_score)
        item.reasons.append(reason)


def __calc_total_score__(order_score, guanxi_score, lushen: str, banlu: [str], gn: GaoNeng):
    total = order_score + guanxi_score
    lushen_param = 0
    if gn.arg1 == lushen or gn.arg2 == lushen:
        lushen_param = 1
    elif gn.arg1 in banlu or gn.arg2 in banlu:
        lushen_param = 0.5

    if lushen_param > 0:
        lushen_score = 0
        if Configs.config_gaoneng_lushen_score.__contains__(gn.guanxi):
            lushen_score = Configs.config_gaoneng_lushen_score[gn.guanxi]
        eft = 1 + lushen_score * lushen_param / 100
        total = round(eft * total, 0)
    return total


def __has_hua_sha__(zhu: SiZhu):
    hasHe = False
    hasSha = False

    for gn in zhu.gaonengs:
        if gn.guanxi in ["合", "暗合"]:
            hasHe = True
        elif gn.guanxi in ["冲", "刑", "穿", "破"]:
            hasSha = True

    return hasHe and hasSha


def __get_guanxi_score__(gn: GaoNeng):
    guanxi_value = Configs.get_value(Configs.config_gaoneng_guanxi_score, gn.guanxi)
    if guanxi_value is None:
        guanxi_value = 0
    return guanxi_value


def __get_order_score__(gn: GaoNeng):
    score = 0
    if gn.guanxi not in ["自刑", "三会"]:
        try:
            score = Configs.config_gaoneng_score[gn.guanxi][gn.shi_shen1][gn.shi_shen2]
        except Exception:
            print("not found gaoneng score:", gn.guanxi, gn.shi_shen1, gn.shi_shen2)
    return score


def __create_qushi_items__(result: {}):
    for key in Configs.config_shishen_list:
        item = QuShiItem()
        item.score = 0
        item.reasons = []
        item.key = key
        result[key] = item
