from yixue import Configs
from yixue.Models import BaZi, PowerCounter, SiZhu, SiZhuResult
from yixue.sizhu import CalcSiZhu, CalcGaoNeng, CalcPower, CheckMuKu, CalcDaYun, CalcShenSha, CalcLiuNian, CalcLiuYue, \
    CalcLiuRi, CalcYinYuan


def calc_si_zhu(year: int, month: int, day: int, know_time: bool, hour: int, minute: int, sheng: str, shi: str,
                qu: str) -> SiZhuResult:
    """
        返回数组，顺序是：[年干、年支、月干、月支、日干、日支、时干、时支]
    """

    # test
    # year = 1989
    # month = 10
    # day = 2
    # hour = 18
    # minute = 30
    # sheng = "福建省"
    # shi = "宁德"
    # qu = "市辖区"
    result = CalcSiZhu.run(year, month, day, know_time, hour, minute, sheng, shi, qu)
    return result


def calc_wu_xing(bazi: BaZi):
    for sz in bazi.all_zhus():
        sz.wu_xing = Configs.get_value(Configs.config_wuxing, sz.text)


def calc_yin_yang(bazi: BaZi):
    for sz in bazi.all_zhus():
        sz.yin_yang = Configs.get_value(Configs.config_yinyang, sz.text)


def calc_shi_shen(bazi: BaZi):
    for sz in bazi.all_zhus():
        sz.shi_shen = Configs.get_value2(Configs.config_shishen, bazi.ri_gan.text, sz.text)


def calc_cang_gans(bazi: BaZi):
    riyuan = bazi.ri_gan.text
    for dz in bazi.all_di_zhis():
        data = Configs.get_value(Configs.config_canggan, dz.text)
        if data is None:
            continue
        dz.cang_gan1 = data[0]
        dz.cang_gan2 = data[1]
        dz.cang_gan3 = data[2]
        dz.gan1_wu_xing = Configs.config_wuxing[dz.cang_gan1]
        dz.gan1_shi_shen = Configs.config_shishen[riyuan][dz.cang_gan1]
        if dz.cang_gan2 is not None:
            dz.gan2_wu_xing = Configs.config_wuxing[dz.cang_gan2]
            dz.gan2_shi_shen = Configs.config_shishen[riyuan][dz.cang_gan2]
        if dz.cang_gan3 is not None:
            dz.gan3_wu_xing = Configs.config_wuxing[dz.cang_gan3]
            dz.gan3_shi_shen = Configs.config_shishen[riyuan][dz.cang_gan3]


def calc_zhang_shengs(bazi: BaZi):
    bazi.nian_zhi.zhang_sheng_zhu = Configs.get_value2(Configs.config_zhangsheng, bazi.nian_gan.text,
                                                       bazi.nian_zhi.text)
    bazi.nian_zhi.zhang_sheng_ri = Configs.get_value2(Configs.config_zhangsheng, bazi.ri_gan.text, bazi.nian_zhi.text)

    bazi.yue_zhi.zhang_sheng_zhu = Configs.get_value2(Configs.config_zhangsheng, bazi.yue_gan.text,
                                                      bazi.yue_zhi.text)
    bazi.yue_zhi.zhang_sheng_ri = Configs.get_value2(Configs.config_zhangsheng, bazi.ri_gan.text, bazi.yue_zhi.text)

    bazi.ri_zhi.zhang_sheng_zhu = Configs.get_value2(Configs.config_zhangsheng, bazi.ri_gan.text,
                                                     bazi.ri_zhi.text)
    bazi.ri_zhi.zhang_sheng_ri = Configs.get_value2(Configs.config_zhangsheng, bazi.ri_gan.text, bazi.ri_zhi.text)

    bazi.shi_zhi.zhang_sheng_zhu = Configs.get_value2(Configs.config_zhangsheng, bazi.shi_gan.text,
                                                      bazi.shi_zhi.text)
    bazi.shi_zhi.zhang_sheng_ri = Configs.get_value2(Configs.config_zhangsheng, bazi.ri_gan.text, bazi.shi_zhi.text)


def calc_gao_nengs(bazi: BaZi):
    riyuan = bazi.ri_gan.text
    tgs = bazi.all_tian_gans()
    for tg1 in tgs:
        for tg2 in tgs:
            if tg1.wei_zhi == tg2.wei_zhi:
                continue
            CalcGaoNeng.fix_gao_neng(riyuan, tg1, tg2, None)

    dzs = bazi.all_di_zhis()
    for dz1 in dzs:
        for dz2 in dzs:
            for dz3 in dzs:
                if dz1.wei_zhi == dz2.wei_zhi or dz2.wei_zhi == dz3.wei_zhi or dz1.wei_zhi == dz3.wei_zhi:
                    continue
                CalcGaoNeng.fix_gao_neng(riyuan, dz1, dz2, dz3)

    for dz1 in dzs:
        for dz2 in dzs:
            if dz1.wei_zhi == dz2.wei_zhi:
                continue
            CalcGaoNeng.fix_gao_neng(riyuan, dz1, dz2, None)


def calc_counter_and_steps(bazi: BaZi):
    bazi.counter = PowerCounter()
    bazi.power_steps = CalcPower.calc_power_step(bazi)
    # counter分几种，原局、高能、墓库
    bazi.counter.power_raw = CalcPower.count_wu_xing_shi_shen(bazi)
    bazi.counter.power_gaoneng = CalcPower.count_diff_of_gaoneng(bazi)
    bazi.counter.power_muku = CalcPower.count_diff_of_muku(bazi)
    bazi.counter.power_zihe = CalcPower.count_diff_of_zihe(bazi)


def calc_wang_shuai(bazi: BaZi):
    tong = 0
    yi = 0
    for key in bazi.counter.power_raw:
        value = bazi.counter.power_raw[key]
        if key in Configs.config_tongdang_list:
            tong += value
        if key in Configs.config_yidang_list:
            yi += value
    bazi.tong_dang = round(tong, 1)
    bazi.yi_dang = round(yi, 1)


def check_muku(bazi: BaZi):
    for dz in bazi.all_di_zhis():  # type: SiZhu
        if dz.text == "无":
            continue
        if dz.is_muku:
            dz.is_muku_open = CheckMuKu.is_muku_open(dz, bazi)


def check_shen_sha(bazi: BaZi):
    CalcShenSha.run(bazi)


def calc_da_yun(bazi: BaZi):
    CalcDaYun.run(bazi)


def calc_liu_nian(bazi: BaZi):
    CalcLiuNian.run(bazi)


def calc_liu_yue(bazi: BaZi):
    CalcLiuYue.run(bazi)


def calc_liu_ri(bazi: BaZi):
    CalcLiuRi.run(bazi)


def calc_dayun_gaoneng(bazi):
    CalcDaYun.calc_gaoneng(bazi)


def calc_dayun_shensha(bazi: BaZi):
    CalcDaYun.calc_shensha(bazi)


def calc_liunian_gaoneng(bazi: BaZi):
    CalcLiuNian.calc_gaonengs(bazi)


def calc_liunian_shensha(bazi: BaZi):
    CalcLiuNian.calc_shensha(bazi)


def calc_liuyue_gaoneng(bazi: BaZi):
    CalcLiuYue.calc_gaoneng(bazi)


def calc_liuyue_shensha(bazi: BaZi):
    CalcLiuYue.calc_shen_sha(bazi)


def calc_liuri_gaoneng(bazi: BaZi):
    CalcLiuRi.calc_gaoneng(bazi)


def calc_liuri_shensha(bazi: BaZi):
    CalcLiuRi.calc_shensha(bazi)


def calc_yinyuan(bazi: BaZi):
    CalcYinYuan.calc_yinyuan(bazi)
