from yixue import Configs
from yixue.Models import BaZi, PowerQiangRuo
from yixue.zhenduan import AboutWuXingShiShenZuoZhi, AboutMenHu, AboutShenSha, AboutGeJu, AboutGaoNeng, AboutJianKang, \
    AboutQingGan, AboutDaYun, AboutYinYuan

wx_per = {}
ss_per = {}
qiang_ruo = PowerQiangRuo()


def calc_power_percentages(bazi: BaZi):
    global wx_per, ss_per, qiang_ruo
    wx_per = __calc_wuxing_power_percentage__(bazi)
    ss_per = __calc_shishen_power_percentage__(bazi)
    qiang_ruo = __calc_qiang_ruo__(bazi)


def about_wuxing_shishen_zuozhi(bazi: BaZi):
    AboutWuXingShiShenZuoZhi.about_wuxing(bazi, wx_per)
    AboutWuXingShiShenZuoZhi.about_shi_shen(bazi, ss_per)
    AboutWuXingShiShenZuoZhi.about_zuozhi(bazi)


def about_menhu(bazi: BaZi):
    AboutMenHu.run(bazi, qiang_ruo)


def about_shen_sha(bazi: BaZi):
    AboutShenSha.run(bazi)


def about_geju(bazi: BaZi):
    AboutGeJu.run(bazi, ss_per)


def about_gaoneng(bazi: BaZi):
    AboutGaoNeng.run(bazi)


def about_jiankang(bazi: BaZi):
    AboutJianKang.run(bazi, wx_per)


def about_qinggan(bazi: BaZi):
    AboutQingGan.run(bazi, qiang_ruo)


def about_dayun(bazi: BaZi):
    AboutDaYun.run(bazi)


def about_yinyuan(bazi: BaZi):
    AboutYinYuan.run(bazi, wx_per, ss_per)


def __calc_wuxing_power_percentage__(bazi: BaZi):
    total = 0
    wx_power = {}
    wx_per = {}
    for wx in Configs.config_wuxing_list:
        step = bazi.power_steps[wx]
        amount = bazi.counter.power_raw[wx]
        power = step * amount
        total += power
        wx_power[wx] = power

    for wx in Configs.config_wuxing_list:
        wx_per[wx] = round(wx_power[wx] * 100 / total, 0)

    return wx_per


def __calc_shishen_power_percentage__(bazi: BaZi):
    total = 0
    ss_power = {}
    ss_per = {}
    keys = [ss for ss in Configs.config_shishen_list]
    for key in ["官杀", "财星", "印绶", "食伤", "比劫"]:
        keys.append(key)

    for key in keys:
        ss_power[key] = 0
        ss_per[key] = 0

    for ss in Configs.config_shishen_list:
        step = bazi.power_steps[ss]
        amount = bazi.counter.power_raw[ss]
        power = step * amount
        total += power
        ss_power[ss] = power
        if ss in ["官", "杀"]:
            ss_power["官杀"] += power
        elif ss in ["财", "才"]:
            ss_power["财星"] += power
        elif ss in ["印", "枭"]:
            ss_power["印绶"] += power
        elif ss in ["食", "伤"]:
            ss_power["食伤"] += power
        elif ss in ["比", "劫"]:
            ss_power["比劫"] += power

    for ss in keys:
        ss_per[ss] = round(ss_power[ss] * 100 / total, 0)
    return ss_per


def __calc_qiang_ruo__(bazi: BaZi):
    result = PowerQiangRuo()
    result.mu_ruo = wx_per["木"] <= 10
    result.huo_ruo = wx_per["火"] <= 10
    result.tu_ruo = wx_per["土"] <= 10
    result.jin_ruo = wx_per["金"] <= 10
    result.shui_ruo = wx_per["水"] <= 10
    result.mu_wang = wx_per["木"] > 25
    result.huo_wang = wx_per["火"] > 25
    result.tu_wang = wx_per["土"] > 25
    result.jin_wang = wx_per["金"] > 25
    result.shui_wang = wx_per["水"] > 25
    result.guan_sha_ruo = ss_per["官杀"] <= 10
    result.cai_xing_ruo = ss_per["财星"] <= 10
    result.yin_shou_ruo = ss_per["印绶"] <= 10
    result.shi_shang_ruo = ss_per["食伤"] <= 10
    result.bi_jie_ruo = ss_per["比劫"] <= 10
    result.guan_sha_wang = ss_per["官杀"] > 25
    result.cai_xing_wang = ss_per["财星"] > 25
    result.yin_shou_wang = ss_per["印绶"] > 25
    result.shi_shang_wang = ss_per["食伤"] > 25
    result.bi_jie_wang = ss_per["比劫"] > 25
    result.guan_sha_guo_wang = ss_per["官杀"] >= 37.5
    result.cai_xing_guo_wang = ss_per["财星"] >= 37.5

    if bazi.ri_zhi.wu_xing == "木":
        result.mu_ruo = False
        result.mu_wang = True
    elif bazi.ri_zhi.wu_xing == "火":
        result.huo_ruo = False
        result.huo_wang = True
    elif bazi.ri_zhi.wu_xing == "土":
        result.tu_ruo = False
        result.tu_wang = True
    elif bazi.ri_zhi.wu_xing == "金":
        result.jin_ruo = False
        result.jin_wang = True
    elif bazi.ri_zhi.wu_xing == "水":
        result.shui_ruo = False
        result.shui_wang = True

    if bazi.ri_zhi.shi_shen in ["官", "杀"]:
        result.guan_sha_ruo = False
        result.guan_sha_wang = True
    elif bazi.ri_zhi.shi_shen in ["财", "才"]:
        result.cai_xing_ruo = False
        result.cai_xing_wang = True
    elif bazi.ri_zhi.shi_shen in ["印", "枭"]:
        result.yin_shou_ruo = False
        result.yin_shou_wang = True
    elif bazi.ri_zhi.shi_shen in ["食", "伤"]:
        result.shi_shang_ruo = False
        result.shi_shang_wang = True
    elif bazi.ri_zhi.shi_shen in ["比", "劫"]:
        result.bi_jie_ruo = False
        result.bi_jie_wang = True

    return result
