from yixue import Configs
from yixue.Models import BaZi, XiongJi


# 要注意，这里其实有2个概念，一个叫天干地支影响，一个叫格局。
# 天干地支影响指的是当10天干12地支到来时，对原局、大运、流年等影响，可以用来看哪些年份、月份的利弊。
# 客户端页面上的天干地支选择指的就是这个，也可以用来看五行应该怎么增补最有利
# 格局指的是八字的日元影响下，高能关系转换为十神会是什么，这里和原局、大运这些没有关系，只是一种由日元
# 决定下的八字的适配性。比如乙木日元，一旦遇到辛，必然七杀攻身。乙木日元，比如酉辰合，促成了七杀合正财，这里跟
# 原局、大运等是否有辰和酉这两个因素没有关系。
# 格局和力量无关，只和日元有关，格局是否触发看大运流年

def run(bazi: BaZi):
    bazi.geju_xiongji = {}
    for ss in Configs.config_shishen_list:
        bazi.geju_xiongji[ss] = []
    __check_guan__(bazi)
    __check_sha__(bazi)
    __check_yin__(bazi)
    __check_xiao__(bazi)
    __check_zheng_cai__(bazi)
    __check_pian_cai__(bazi)
    __check_shi_shen__(bazi)
    __check__jie_cai__(bazi)


def __check_guan__(bazi: BaZi):
    # 检查伤官冲官，劫财冲官，官暗合财，官印相生
    name1 = "伤官冲官"
    tip1 = "行事容易越界，冲撞、挑战权威，也善于攻克难关。"
    __check_gaoneng_in_bazi__(name1, "官", "伤", ["冲", "穿"], bazi, tip1)

    name2 = "劫财冲官"
    tip2 = "与人合作要注意行事不要越界，也可能涉及动到行业内他人的利益。"
    __check_gaoneng_in_bazi__(name2, "官", "劫", ["冲", "穿"], bazi, tip2)

    name3 = "暗合财星"
    tip3 = "要注意行为的正当性，尤其是一些不上台面的交易。"
    found = __check_gaoneng_in_bazi__(name3, "官", "财", ["暗合"], bazi, tip3)
    if not found:
        __check_gaoneng_in_bazi__(name3, "官", "才", ["暗合"], bazi, tip3)
    name4 = "官印相生"
    tip4 = "有权有势，容易得名望得地位。"
    found = __check_gaoneng_in_bazi__(name4, "官", "印", ["合", "暗合"], bazi, tip4)
    if not found:
        __check_gaoneng_in_bazi__(name4, "官", "枭", ["合", "暗合"], bazi, tip4)
    name5 = "官制劫财"
    tip5 = "保护自身财富不被掠夺，能克制小人。"
    __check_exist_in_bazi__(name5, "官", "劫", bazi, tip5)


def __check_sha__(bazi: BaZi):
    # 检查七杀冲比肩，七杀合劫财，七杀合伤官，杀印相生
    name1 = "七杀攻身"
    tip1 = "容易被诸多事务缠身，压力大，影响健康。"
    __check_gaoneng_in_bazi__(name1, "杀", "比", ["冲", "合", "穿", "暗合"], bazi, tip1)
    name2 = "七杀劫财"
    tip2 = "事务缠身，处处救火。处处都被需要，疲于奔命。"
    __check_gaoneng_in_bazi__(name2, "杀", "劫", ["合", "暗合"], bazi, tip2)
    name3 = "七杀伤官"
    tip3 = "容易挑战超过自己能力的事情。"
    __check_gaoneng_in_bazi__(name3, "杀", "伤", ["合", "暗合"], bazi, tip3)
    name4 = "杀印相生"
    tip4 = "容易得贵人帮助，带来巨大机会。"
    found = __check_gaoneng_in_bazi__(name4, "杀", "印", ["合", "暗合"], bazi, tip4)
    if not found:
        __check_gaoneng_in_bazi__(name4, "杀", "枭", ["合", "暗合"], bazi, tip4)


def __check_yin__(bazi: BaZi):
    # 检查食伤合印，财损印
    name1 = "食伤合印"
    tip1 = "学习能力强，并能迅速转化应用，计划的事情都能一步步去实现。"
    found = __check_gaoneng_in_bazi__(name1, "印", "食", ["合", "暗合"], bazi, tip1)
    if not found:
        found = __check_gaoneng_in_bazi__(name1, "印", "伤", ["合", "暗合"], bazi, tip1)
        if not found:
            found = __check_gaoneng_in_bazi__(name1, "枭", "食", ["合", "暗合"], bazi, tip1)
            if not found:
                __check_gaoneng_in_bazi__(name1, "枭", "伤", ["合", "暗合"], bazi, tip1)
    name2 = "财小印大"
    tip2 = "用小的代价想得到大的回报，要成事需要投入更多的精力、资源。印容易让人犯懒，不谋事还好，一旦要谋事，务必投入更多时间精力资源才可成事。"
    __compile_power__(name2, ["印", "枭"], ["财", "才"], "印", "财", 0.5, bazi, tip2)

    name3 = "印禄相随"
    tip3 = "容易得贵人相帮，东山再起。"
    if __has__lu_shen__(bazi):
        found = __check_exist_in_bazi__(name3, "印", None, bazi, tip3)
        if not found:
            __check_exist_in_bazi__(name3, "枭", None, bazi, tip3)


def __check_xiao__(bazi: BaZi):
    # 检查枭神夺食
    name1 = "枭神夺食"
    tip1 = "被约束，有口难言，不得志。"
    __check_exist_in_bazi__(name1, "枭", "食", bazi, tip1)


def __check_zheng_cai__(bazi: BaZi):
    __check_gaoneng_in_bazi__("合官", "财", "官", ["合", "暗合", "冲"], bazi, "名望地位带来财富。")
    __check_gaoneng_in_bazi__("合杀", "财", "杀", ["合", "暗合", "冲"], bazi, "社会资源带来财富。")
    __check_gaoneng_in_bazi__("合印", "财", "印", ["合", "暗合", "冲"], bazi, "贵人事业带来财富。")
    __check_gaoneng_in_bazi__("合枭", "财", "枭", ["合", "暗合", "冲"], bazi, "贵人事业带来财富。")
    __check_gaoneng_in_bazi__("合食", "财", "食", ["合", "暗合", "冲"], bazi, "知识技能带来财富。")
    __check_gaoneng_in_bazi__("合伤", "财", "伤", ["合", "暗合", "冲"], bazi, "知识技能带来财富。")
    __check_gaoneng_in_bazi__("合比", "财", "比", ["合", "暗合", "冲"], bazi, "朋友协作带来财富。")
    __check_gaoneng_in_bazi__("合劫", "财", "劫", ["合", "暗合", "冲"], bazi, "朋友协作带来财富。")


def __check_pian_cai__(bazi: BaZi):
    __check_gaoneng_in_bazi__("合官", "才", "官", ["合", "暗合", "冲"], bazi, "名望地位带来财富。")
    __check_gaoneng_in_bazi__("合杀", "才", "杀", ["合", "暗合", "冲"], bazi, "社会资源带来财富。")
    __check_gaoneng_in_bazi__("合印", "才", "印", ["合", "暗合", "冲"], bazi, "贵人事业带来财富。")
    __check_gaoneng_in_bazi__("合枭", "才", "枭", ["合", "暗合", "冲"], bazi, "贵人事业带来财富。")
    __check_gaoneng_in_bazi__("合食", "才", "食", ["合", "暗合", "冲"], bazi, "知识技能带来财富。")
    __check_gaoneng_in_bazi__("合伤", "才", "伤", ["合", "暗合", "冲"], bazi, "知识技能带来财富。")
    __check_gaoneng_in_bazi__("合比", "才", "比", ["合", "暗合", "冲"], bazi, "朋友协作带来财富。")
    __check_gaoneng_in_bazi__("合劫", "才", "劫", ["合", "暗合", "冲"], bazi, "朋友协作带来财富。")


def __check_shi_shen__(bazi: BaZi):
    __check_exist_in_bazi__("食神制杀", "食", "杀", bazi, "能保护自身，避开小人。")


def __check__jie_cai__(bazi: BaZi):
    found = __check_exist_in_bazi__("劫财见财", "劫", "财", bazi, "财富容易被掠夺。")
    if not found:
        __check_exist_in_bazi__("劫财见财", "劫", "才", bazi, "财富容易被掠夺。")


def __compile_power__(name: str, shishenBiggers: [str], shishenLowers: [str], key1: str, key2: str, percentage: float,
                      bazi: BaZi, tip: str):
    p1 = 0
    for ss in shishenBiggers:
        step = bazi.power_steps[ss]
        value = bazi.counter.power_raw[ss]
        p1 += step * value

    p2 = 0
    for ss in shishenLowers:
        step = bazi.power_steps[ss]
        value = bazi.counter.power_raw[ss]
        p2 += step * value

    if p1 * percentage > p2:
        xj = XiongJi()
        xj.name = name
        xj.arg1 = key1
        xj.arg2 = key2
        xj.guanxi = ""
        xj.tip = tip
        bazi.geju_xiongji[key1].append(xj)


def __has__lu_shen__(bazi: BaZi):
    found = False
    for dz in bazi.all_di_zhis():
        if "禄神" in dz.shenshas:
            found = True
            break
        elif "半禄" in dz.shenshas:
            found = True
            break
    return found


def __check_exist_in_bazi__(name: str, key1: str, key2: str, bazi: BaZi, tip: str):
    ary1 = bazi.shishen_guanxi[key1]
    has1 = False
    for item1 in ary1:
        if __is_in_bazi__(item1, bazi):
            has1 = True
        if has1:
            break

    has2 = False
    if key2 is None:
        has2 = True
    else:
        ary2 = bazi.shishen_guanxi[key2]
        for item2 in ary2:
            if __is_in_bazi__(item2, bazi):
                has2 = True
            if has2:
                break

    if has1 and has2:
        xj = XiongJi()
        xj.name = name
        xj.arg1 = key1
        xj.arg2 = key2
        xj.guanxi = ""
        xj.tip = tip
        bazi.geju_xiongji[key1].append(xj)
    return has1 and has2


def __build_unfound_bean__(name: str, key: str, tip: str, bazi: BaZi):
    xj = XiongJi()
    xj.name = name
    xj.found = False
    xj.tip = tip
    bazi.geju_xiongji[key].append(xj)


def __check_gaoneng_in_bazi__(name: str, key1: str, key2: str, guanxis: [str], bazi: BaZi, tip: str):
    ary1 = bazi.shishen_guanxi[key1]
    ary2 = bazi.shishen_guanxi[key2]
    found = False
    for item1 in ary1:
        for item2 in ary2:
            key = item1 + item2
            if Configs.config_gaoneng_query.__contains__(key):
                gns = Configs.config_gaoneng_query[key]
                for gn in gns:
                    if gn["guanxi"] in guanxis:
                        xj = XiongJi()
                        xj.name = name
                        xj.arg1 = key1
                        xj.arg2 = key2
                        xj.guanxi = gn["guanxi"]
                        xj.tip = tip
                        xj.found = True
                        bazi.geju_xiongji[key1].append(xj)
                        found = True
                        break
            if found:
                break
        if found:
            break
    return found


def __is_in_bazi__(key: str, bazi: BaZi):
    if bazi.nian_gan.text == key:
        return True
    if bazi.nian_zhi.text == key:
        return True
    if bazi.yue_gan.text == key:
        return True
    if bazi.yue_zhi.text == key:
        return True
    if bazi.ri_gan.text == key:
        return True
    if bazi.ri_zhi.text == key:
        return True
    if bazi.shi_gan.text == key:
        return True
    if bazi.shi_zhi.text == key:
        return True
    return False
