import datetime
import math

from wx.dbTools import DBDaYunNan, DBDaYunNv
from yixue import Configs
from yixue.Models import BaZi, DaYun, SiZhu
from yixue.sizhu import CalcGaoNeng, CalcShenSha


def run(bazi: BaZi):
    qiyun_date = __calc_qi_yun__(bazi.is_man, bazi.tai_yang_shi)
    print("起运时间：", qiyun_date)
    if qiyun_date is not None:
        dayun_orders = __order_dayun__(bazi)
        __build_dayuns__(bazi, qiyun_date, dayun_orders)
    else:
        bazi.dayun = []


def calc_gaoneng(bazi: BaZi):
    tgs = bazi.all_tian_gans()
    dzs = bazi.all_di_zhis()
    riyuan = bazi.ri_gan.text
    for dy in bazi.dayun:
        if not dy.is_dayun:
            continue

        # check tian gan
        for tg in tgs:
            CalcGaoNeng.fix_gao_neng(riyuan, dy.gan_zhu, tg, None)
            CalcGaoNeng.fix_gao_neng(riyuan, tg, dy.gan_zhu, None, dy.trigger)

        for dz1 in dzs:
            for dz2 in dzs:
                if dz1.wei_zhi == dz2.wei_zhi:
                    continue
                CalcGaoNeng.fix_gao_neng(riyuan, dy.zhi_zhu, dz1, dz2)
                CalcGaoNeng.fix_gao_neng(riyuan, dz1, dz2, dy.zhi_zhu, dy.trigger)
                CalcGaoNeng.fix_gao_neng(riyuan, dz2, dy.zhi_zhu, dz1, dy.trigger)

        for dz in dzs:
            CalcGaoNeng.fix_gao_neng(riyuan, dy.zhi_zhu, dz, None)
            CalcGaoNeng.fix_gao_neng(riyuan, dz, dy.zhi_zhu, None, dy.trigger)


def calc_shensha(bazi: BaZi):
    riyuan = bazi.ri_gan.text
    for dy in bazi.dayun:
        if not dy.is_dayun:
            continue

        CalcShenSha.__check_lu_shen__(dy.zhi_zhu, riyuan)
        CalcShenSha.__check_yang_ren__(dy.zhi_zhu, riyuan)
        CalcShenSha.__check_wen_chang__(dy.gan_zhu, dy.zhi_zhu)
        CalcShenSha.__check_kui_gang__(dy.gan_zhu, dy.zhi_zhu)
        CalcShenSha.__check_yi_ma__(bazi.nian_zhi, bazi.ri_zhi, [dy.zhi_zhu])
        CalcShenSha.__check_tianluo_diwang__(bazi.nian_zhi, bazi.ri_zhi, [dy.zhi_zhu])
        CalcShenSha.__check_tao_hua__(bazi.nian_zhi, bazi.ri_zhi, [dy.zhi_zhu])


def __create_temp_sizhu__(text: str, riyuan: str, weizhi: int):
    sz = SiZhu(text, weizhi)
    sz.wu_xing = Configs.get_value(Configs.config_wuxing, text)
    sz.shi_shen = Configs.get_value2(Configs.config_shishen, riyuan, text)
    return sz


def __calc_qi_yun__(is_man: bool, tai_yang_shi: datetime):
    if tai_yang_shi is None:  # 未知时辰的时候，太阳时是没有的
        return
    # tai_yang_shi = datetime.datetime(1910, 6, 20, 13, 40, 0)
    # is_man = False
    table = DBDaYunNan
    if not is_man:
        table = DBDaYunNv

    ss = table.new_session()
    dt = ss.query(table).filter(table.sign is not None, table.time < tai_yang_shi).order_by(
        table.time.desc()).first()
    # print("最近的参考时间：", dt)
    result = None
    if dt is not None:
        # 这里有两个概念，一个是起运时间(qi_yun)，一个是现实时间(real)
        # 起运时间是几个整数组成的，包括年、月、日、时，它不代表一个具体的时间，只是时间差的一种标记
        # 起运时间是24时=1日，30日=1月，12月=1年
        # 起运值 = ((起运年 *12 + 起运月) * 30 + 起运日) * 24 + 起运时
        # 现实时间过1分钟，大运时间差2小时（可能加，可能减，看符号）
        # 比如：现实时间 00:29,对应大运时间是 2天6时；现实时间00:30，对应大运时间是2天4时（举个例子）

        if dt.sign == "-":
            # 如果符号是-，就以当前参考值的时间为基准，计算两者时间差，并换算成分钟(/60)
            real_min_diff = math.trunc((tai_yang_shi - dt.time).total_seconds() / 60)
            qiyun_min_diff = real_min_diff * 2  # 换算成大运的差异，要乘以2
            # 把数据库中提取的参考时间，换算成起运总值,符号是-的话，是以这个数值为基准开始减
            qiyun_value = ((dt.year * 12 + dt.month) * 30 + dt.day) * 24 + dt.hour
            # 两个基于相同纬度的值差，就是目标日期和参考日期的起运值差异
            qiyun_diff = abs(qiyun_min_diff - qiyun_value)
            # 把总的起运值差异，换算成年、月、日、时
            result = __calc_qiyun_date__(qiyun_diff)
            # print("起运:", result)
        elif dt.sign == "+":
            # 如果符号是-，就以当前参考值的时间为基准，计算两者时间差，并换算成分钟(/60)
            real_min_diff = math.trunc((tai_yang_shi - dt.time).total_seconds() / 60)
            qiyun_min_diff = real_min_diff * 2  # 换算成大运的差异，要乘以2
            # 把数据库中提取的参考时间，换算成起运总值,符号是+的话，是以这个数值为基准开始加
            qiyun_value = ((dt.year * 12 + dt.month) * 30 + dt.day) * 24 + dt.hour
            # 两个基于相同纬度的值差，就是目标日期和参考日期的起运值差异
            qiyun_diff = abs(qiyun_min_diff - qiyun_value)
            # 把总的起运值差异，换算成年、月、日、时
            result = __calc_qiyun_date__(qiyun_diff)
            # print("起运:", result)
    else:
        print("__calc_qi_yun__，未找到合适的参考时间：", tai_yang_shi)
    ss.close()
    return result


def __calc_qiyun_date__(total: int):
    year_cost = 8640
    month_cost = 720
    day_cost = 24

    year = math.trunc(total / year_cost)
    remain = total - year * year_cost
    month = math.trunc(remain / month_cost)
    remain = remain - month * month_cost
    day = math.trunc(remain / day_cost)
    remain = remain - day * day_cost
    hour = remain
    return [year, month, day, hour]


def __order_dayun__(bazi: BaZi):
    # 排10个大运，大运排法看年干定阴阳，再看性别
    # 以月干为基准，阳男阴女顺排，阴男阳女逆排
    gan = bazi.yue_gan.text
    zhi = bazi.yue_zhi.text

    is_man = bazi.is_man
    yin_yang = bazi.nian_gan.yin_yang

    dayun = []
    step = 1
    if is_man:
        if yin_yang == "阳":
            step = 1
        else:
            step = -1
    else:
        if yin_yang == "阳":
            step = -1
        else:
            step = 1

    for index in range(0, 10):
        gan = __next_tiangan__(gan, step)
        zhi = __next_dizhi__(zhi, step)
        dayun.append([gan, zhi])
    return dayun


def __build_dayuns__(bazi: BaZi, qiyun_date: [int], dayun_orders: [str]):
    # 把起运数据换算成天
    days = (qiyun_date[0] * 12 + qiyun_date[1]) * 30
    new_date: datetime.datetime = datetime.timedelta(days=days) + bazi.tai_yang_shi
    qiyun_year = new_date.year
    qiyun_age = new_date.year - bazi.tai_yang_shi.year + 1

    bazi.dayun = []
    # 处理小运
    xiaoyun = DaYun()
    xiaoyun.is_dayun = False
    xiaoyun.gan_text = "小"
    xiaoyun.zhi_text = "运"
    xiaoyun.start_age = 1
    xiaoyun.end_age = qiyun_age - 1
    xiaoyun.start_year = bazi.tai_yang_shi.year
    xiaoyun.end_year = qiyun_year - 1
    xiaoyun.gan_zhu = SiZhu("小", Configs.const_dayun_gan)
    xiaoyun.zhi_zhu = SiZhu("运", Configs.const_dayun_zhi)
    xiaoyun.gan_zhu.dayun_text = "小运"
    xiaoyun.zhi_zhu.dayun_text = "小运"
    xiaoyun.trigger = xiaoyun.gan_text + xiaoyun.zhi_text

    bazi.dayun.append(xiaoyun)

    # 处理大运
    riyuan = bazi.ri_gan.text
    lu_shen: str = Configs.get_value2(Configs.config_lushen, bazi.ri_gan.text, "禄神")
    ban_lu: [str] = Configs.get_value2(Configs.config_lushen, bazi.ri_gan.text, "半禄")
    for (index, dy) in enumerate(dayun_orders):
        dayun = DaYun()
        dayun.is_dayun = True
        dayun.gan_text = dy[0]
        dayun.zhi_text = dy[1]
        dayun.trigger = dayun.gan_text + dayun.zhi_text
        dayun.start_age = qiyun_age + index * 10
        dayun.end_age = dayun.start_age + 10 - 1
        dayun.start_year = qiyun_year + index * 10
        dayun.end_year = dayun.start_year + 10 - 1

        gan = SiZhu(dy[0], Configs.const_dayun_gan)
        __fill_sizhu_data(gan, riyuan, False)
        zhi = SiZhu(dy[1], Configs.const_dayun_zhi)
        __fill_sizhu_data(zhi, riyuan, True)
        dayun.gan_zhu = gan
        dayun.zhi_zhu = zhi
        dayun.gan_zhu.dayun_text = dayun.gan_text + dayun.zhi_text
        dayun.zhi_zhu.dayun_text = dayun.gan_text + dayun.zhi_text

        bazi.dayun.append(dayun)


def __fill_sizhu_data(sz: SiZhu, riyuan: str, is_dizhi: bool):
    sz.wu_xing = Configs.get_value(Configs.config_wuxing, sz.text)
    sz.shi_shen = Configs.get_value2(Configs.config_shishen, riyuan, sz.text)
    if is_dizhi:
        cang_gan = Configs.get_value(Configs.config_canggan, sz.text)
        sz.cang_gan1 = cang_gan[0]
        sz.cang_gan2 = cang_gan[1]
        sz.cang_gan3 = cang_gan[2]
        if sz.cang_gan1 is not None:
            sz.gan1_wu_xing = Configs.get_value(Configs.config_wuxing, sz.cang_gan1)
            sz.gan1_shi_shen = Configs.get_value2(Configs.config_shishen, riyuan, sz.cang_gan1)
        if sz.cang_gan2 is not None:
            sz.gan2_wu_xing = Configs.get_value(Configs.config_wuxing, sz.cang_gan2)
            sz.gan2_shi_shen = Configs.get_value2(Configs.config_shishen, riyuan, sz.cang_gan2)
        if sz.cang_gan3 is not None:
            sz.gan3_wu_xing = Configs.get_value(Configs.config_wuxing, sz.cang_gan3)
            sz.gan3_shi_shen = Configs.get_value2(Configs.config_shishen, riyuan, sz.cang_gan3)


def __get_next_by_dist__(ary: [str], item: str, dist: int):
    result = None
    if ary.__contains__(item):
        index = ary.index(item)
        index += dist
        ln = len(ary)
        while index >= ln:
            index -= ln
        result = ary[index]
    return result


def __next_tiangan__(tiangan: str, dist: int):
    return __get_next_by_dist__(Configs.config_tiangan_list, tiangan, dist)


def __next_dizhi__(dizhi: str, dist: int):
    return __get_next_by_dist__(Configs.config_dizhi_list, dizhi, dist)
