from yixue import Configs
from yixue.Models import BaZi, DaYun, LiuNian, SiZhu
from yixue.sizhu import CalcGaoNeng, CalcShenSha


def run(bazi: BaZi):
    for dy in bazi.dayun:
        __calc_liu_nian__(dy, bazi)


def calc_gaonengs(bazi: BaZi):
    tgs = bazi.all_tian_gans()
    dzs = bazi.all_di_zhis()
    riyuan = bazi.ri_gan.text
    for dayun in bazi.dayun:
        for ln in dayun.liunian:

            # 流年天干 vs 原局天干
            for tg in tgs:
                CalcGaoNeng.fix_gao_neng(riyuan, ln.gan_zhu, tg, None)
                CalcGaoNeng.fix_gao_neng(riyuan, tg, ln.gan_zhu, None, ln.trigger)

            # 流年地支 vs 原局地支 3
            for dz1 in dzs:
                for dz2 in dzs:
                    if dz1.wei_zhi == dz2.wei_zhi:
                        continue
                    CalcGaoNeng.fix_gao_neng(riyuan, ln.zhi_zhu, dz1, dz2, )
                    CalcGaoNeng.fix_gao_neng(riyuan, dz1, dz2, ln.zhi_zhu, ln.trigger)
                    CalcGaoNeng.fix_gao_neng(riyuan, dz2, ln.zhi_zhu, dz1, ln.trigger)

            # 流年地支 vs 原局地支 2
            for dz in dzs:
                CalcGaoNeng.fix_gao_neng(riyuan, ln.zhi_zhu, dz, None)
                CalcGaoNeng.fix_gao_neng(riyuan, dz, ln.zhi_zhu, None, ln.trigger)

            # 流年如果来自大运，还要和大运再做一次比对
            if ln.from_dayun:
                other_tgs = [dayun.gan_zhu]
                other_dzs = [dayun.zhi_zhu]

                # 流年天干 vs 大运天干
                for tg in other_tgs:
                    CalcGaoNeng.fix_gao_neng(riyuan, ln.gan_zhu, tg, None)
                    CalcGaoNeng.fix_gao_neng(riyuan, tg, ln.gan_zhu, None, ln.trigger)

                # 流年地支 vs 大运地支 3
                for dz1 in other_dzs:
                    for dz2 in dzs:
                        CalcGaoNeng.fix_gao_neng(riyuan, ln.zhi_zhu, dz1, dz2)
                        CalcGaoNeng.fix_gao_neng(riyuan, dz1, dz2, ln.zhi_zhu, ln.trigger)
                        CalcGaoNeng.fix_gao_neng(riyuan, dz2, ln.zhi_zhu, dz1, ln.trigger)

                # 流年地支 vs 大运地支 2
                for dz in other_dzs:
                    CalcGaoNeng.fix_gao_neng(riyuan, ln.zhi_zhu, dz, None)
                    CalcGaoNeng.fix_gao_neng(riyuan, dz, ln.zhi_zhu, None, ln.trigger)


def calc_shensha(bazi: BaZi):
    riyuan = bazi.ri_gan.text
    for dayun in bazi.dayun:
        for ln in dayun.liunian:
            CalcShenSha.__check_lu_shen__(ln.zhi_zhu, riyuan)
            CalcShenSha.__check_yang_ren__(ln.zhi_zhu, riyuan)
            CalcShenSha.__check_wen_chang__(ln.gan_zhu, ln.zhi_zhu)
            CalcShenSha.__check_kui_gang__(ln.gan_zhu, ln.zhi_zhu)
            CalcShenSha.__check_yi_ma__(bazi.nian_zhi, bazi.ri_zhi, [ln.zhi_zhu])
            CalcShenSha.__check_tianluo_diwang__(bazi.nian_zhi, bazi.ri_zhi, [ln.zhi_zhu])
            CalcShenSha.__check_tao_hua__(bazi.nian_zhi, bazi.ri_zhi, [ln.zhi_zhu])


def __calc_liu_nian__(dayun: DaYun, bazi: BaZi):
    index = 0
    riyuan = bazi.ri_gan.text
    from_gan = bazi.nian_gan.text
    from_zhi = bazi.nian_zhi.text

    # 要判断过没过除夕，没过除夕的话，干支要倒退一位
    # from 干支是从数据库提取的，如果没过除夕，其实对应的是上一年的干支
    # 而出生年 start_year 又是指的当年，所以这种情况下干支会落后一个位，所以要往前移动一位
    go_next = False
    jie_qi = Configs.get_jie_qi(bazi.tai_yang_shi.year, "除夕")
    if jie_qi is not None:
        if bazi.tai_yang_shi.month < jie_qi[0]:
            go_next = True
        elif bazi.tai_yang_shi.month == jie_qi[0]:
            if bazi.tai_yang_shi.day < jie_qi[1]:
                go_next = True

    if go_next:
        from_gan = __next_tiangan__(from_gan, 1)
        from_zhi = __next_dizhi__(from_zhi, 1)

    dayun.liunian = []
    for year in range(dayun.start_year, dayun.end_year + 1):
        dist = year - bazi.tai_yang_shi.year
        ln = LiuNian()
        ln.year = year
        ln.from_dayun = dayun.is_dayun
        ln.age = dayun.start_age + index
        ln.gan_text = __next_tiangan__(from_gan, dist)
        ln.zhi_text = __next_dizhi__(from_zhi, dist)
        ln.trigger = dayun.gan_text + dayun.zhi_text + ln.gan_text + ln.zhi_text

        gan = SiZhu(ln.gan_text, Configs.const_liunian_gan)
        __fill_sizhu_data(gan, riyuan, False)
        zhi = SiZhu(ln.zhi_text, Configs.const_liunian_zhi)
        __fill_sizhu_data(zhi, riyuan, True)
        ln.gan_zhu = gan
        ln.zhi_zhu = zhi
        ln.gan_zhu.liunian_text = ln.gan_text + ln.zhi_text
        ln.zhi_zhu.liunian_text = ln.gan_text + ln.zhi_text
        ln.gan_zhu.dayun_text = dayun.gan_text + dayun.zhi_text
        ln.zhi_zhu.dayun_text = dayun.gan_text + dayun.zhi_text
        index += 1

        dayun.liunian.append(ln)


def __fill_sizhu_data(sz: SiZhu, riyuan: str, is_dizhi: bool):
    sz.wu_xing = Configs.get_value(Configs.config_wuxing, sz.text)
    sz.shi_shen = Configs.get_value2(Configs.config_shishen, riyuan, sz.text)
    if is_dizhi:
        cang_gan = Configs.get_value(Configs.config_canggan, sz.text)
        sz.cang_gan1 = cang_gan[0]
        sz.cang_gan2 = cang_gan[1]
        sz.cang_gan3 = cang_gan[2]
        if sz.cang_gan1 is not None:
            sz.gan1_wu_xing = Configs.get_value(Configs.config_wuxing, sz.cang_gan1)
            sz.gan1_shi_shen = Configs.get_value2(Configs.config_shishen, riyuan, sz.cang_gan1)
        if sz.cang_gan2 is not None:
            sz.gan2_wu_xing = Configs.get_value(Configs.config_wuxing, sz.cang_gan2)
            sz.gan2_shi_shen = Configs.get_value2(Configs.config_shishen, riyuan, sz.cang_gan2)
        if sz.cang_gan3 is not None:
            sz.gan3_wu_xing = Configs.get_value(Configs.config_wuxing, sz.cang_gan3)
            sz.gan3_shi_shen = Configs.get_value2(Configs.config_shishen, riyuan, sz.cang_gan3)


def __get_next_by_dist__(ary: [str], item: str, dist: int):
    result = None
    if ary.__contains__(item):
        index = ary.index(item)
        index += dist
        ln = len(ary)
        while index >= ln:
            index -= ln
        result = ary[index]
    return result


def __next_tiangan__(tiangan: str, dist: int):
    return __get_next_by_dist__(Configs.config_tiangan_list, tiangan, dist)


def __next_dizhi__(dizhi: str, dist: int):
    return __get_next_by_dist__(Configs.config_dizhi_list, dizhi, dist)
