from yixue import Configs
from yixue.Models import BaZi, PowerCounter, GaoNeng, SiZhu


def __get_wu_xing_of_shi_shen(ss: str, bazi: BaZi):
    wx = bazi.ri_gan.wu_xing
    return Configs.get_value2(Configs.config_wuxing_shishen_query, wx, ss)


def calc_power_step(bazi: BaZi):
    """
    返回一个对象，key是五行和十神简称，value是power step
    """
    ary = Configs.congfig_wangshuai[bazi.yue_zhi.wu_xing]
    steps = {
        ary[0]: 100,
        ary[1]: 80,
        ary[2]: 60,
        ary[3]: 40,
        ary[4]: 20,
    }

    bazi.shishen_guanxi = {
        "木": [], "火": [], "土": [], "金": [], "水": [],
    }
    temp_ss_dict = {}
    for ss in Configs.config_shishen_list:
        ss_wx = __get_wu_xing_of_shi_shen(ss, bazi)
        if ss_wx is None:
            continue
        bazi.shishen_guanxi[ss_wx].append(ss)
        value = steps[ss_wx]
        steps[ss] = value
        temp_ss_dict[ss] = []

    for tg in Configs.config_tiangan_list:
        tg_ss = Configs.config_shishen[bazi.ri_gan.text][tg]
        bazi.shishen_guanxi[tg] = tg_ss
        temp_ss_dict[tg_ss].append(tg)
    for dz in Configs.config_dizhi_list:
        dz_ss = Configs.config_shishen[bazi.ri_gan.text][dz]
        bazi.shishen_guanxi[dz] = dz_ss
        temp_ss_dict[dz_ss].append(dz)

    for key in temp_ss_dict:
        bazi.shishen_guanxi[key] = temp_ss_dict[key]
    bazi.shishen_guanxi['禄神'] = Configs.config_lushen[bazi.ri_gan.text]["禄神"]
    bazi.shishen_guanxi['半禄'] = Configs.config_lushen[bazi.ri_gan.text]["半禄"]
    return steps


def count_wu_xing_shi_shen(bazi: BaZi):
    counter = PowerCounter.new_counter()
    for sz in bazi.all_zhus():
        if sz.text == "无":
            continue
        if sz.wei_zhi <= Configs.const_shi_gan:
            counter[sz.wu_xing] += 1
            counter[sz.shi_shen] += 1
        else:
            if sz.cang_gan3 is None:
                if sz.cang_gan2 is None:
                    counter[sz.gan1_wu_xing] += 1
                    counter[sz.gan1_shi_shen] += 1
                else:
                    counter[sz.gan1_wu_xing] += 0.7
                    counter[sz.gan2_wu_xing] += 0.3
                    counter[sz.gan1_shi_shen] += 0.7
                    counter[sz.gan2_shi_shen] += 0.3
            else:
                counter[sz.gan1_wu_xing] += 0.6
                counter[sz.gan2_wu_xing] += 0.3
                counter[sz.gan3_wu_xing] += 0.1
                counter[sz.gan1_shi_shen] += 0.6
                counter[sz.gan2_shi_shen] += 0.3
                counter[sz.gan3_shi_shen] += 0.1

    for key in counter.keys():
        counter[key] = round(counter[key], 1)
    return counter


def count_diff_of_gaoneng(bazi: BaZi):
    riyuan = bazi.ri_gan.text
    # 影响力量分布的高能关系主要是拱合、半合、三会(不稳定，先不看)
    counter = PowerCounter.new_counter()
    for dz in bazi.all_di_zhis():  # type: SiZhu
        for gn in dz.gaonengs:  # type:GaoNeng
            if gn.guanxi in ["半合", "拱合"]:
                target = gn.fangxiang
                if target is not None:
                    wx = Configs.config_wuxing[target]
                    ss = Configs.config_shishen[riyuan][target]
                    counter[wx] += 0.5
                    counter[ss] += 0.5
    return counter


def count_diff_of_muku(bazi: BaZi):
    # 墓库的影响主要是开闭库状态下，藏干中主气中气余气的转换
    counter = PowerCounter.new_counter()
    for dz in bazi.all_di_zhis():  # type:SiZhu
        if dz.is_muku:
            counter[dz.gan1_wu_xing] -= 0.5
            counter[dz.gan1_shi_shen] -= 0.5
            counter[dz.gan2_wu_xing] += 0.3
            counter[dz.gan2_shi_shen] += 0.3
            counter[dz.gan3_wu_xing] += 0.2
            counter[dz.gan3_shi_shen] += 0.2

    return counter


def count_diff_of_zihe(bazi: BaZi):
    counter = PowerCounter.new_counter()
    
    return counter
