from yixue import Configs
from yixue.Models import BaZi, SiZhu


def run(bazi: BaZi):
    riyuan = bazi.ri_gan.text
    tgs = bazi.all_tian_gans()
    dzs = bazi.all_di_zhis()
    for dz in dzs:
        if dz.text == "无":
            continue
        __check_lu_shen__(dz, riyuan)
        __check_yang_ren__(dz, riyuan)
        __check_jiang_xing__(dz)

    for (index, tg) in enumerate(tgs):
        dz = dzs[index]
        __check_wen_chang__(tg, dz)
        __check_kui_gang__(tg, dz)

    __check_yi_ma__(bazi.nian_zhi, bazi.ri_zhi, dzs)
    __check_tianluo_diwang__(bazi.nian_zhi, bazi.ri_zhi, dzs)
    __check_tao_hua__(bazi.nian_zhi, bazi.ri_zhi, dzs)


def __check_lu_shen__(sz: SiZhu, riyuan: str):
    # 以日元查四柱
    info = Configs.config_lushen[riyuan]
    if sz.text == info["禄神"]:
        sz.shenshas.append("禄神")
    elif sz.text in info["半禄"]:
        sz.shenshas.append("半禄")


def __check_yang_ren__(sz: SiZhu, riyuan: str):
    key = "羊刃"
    if sz.text == Configs.config_yangren[riyuan]:
        sz.shenshas.append(key)


def __check_wen_chang__(tg: SiZhu, dz: SiZhu):
    # 天干食神，地支符合文昌配置（以同柱天干为key）
    if tg.shi_shen == "食":
        if dz.text == Configs.config_wenchang[tg.text]:
            dz.shenshas.append("文昌")


def __check_kui_gang__(tg: SiZhu, dz: SiZhu):
    # 天干地支同时满足条件为魁罡
    key = tg.text + dz.text
    if key in Configs.config_kuigang:
        dz.shenshas.append("魁罡")


def __check_yi_ma__(nian_zhi: SiZhu, ri_zhi: SiZhu, dzs: [SiZhu]):
    # 用年支、日支查其余支
    targets = [nian_zhi, ri_zhi]
    for target in targets:
        for dz in dzs:
            if dz == target:
                continue
            if dz.text == "无":
                continue
            if dz.text == Configs.config_yima[target.text]:
                dz.shenshas.append("驿马")


def __check_jiang_xing__(dz: SiZhu):
    if dz.text in ["子", "午", "卯", "酉"]:
        dz.shenshas.append("将星")


def __check_tianluo_diwang__(nian_zhi: SiZhu, ri_zhi: SiZhu, dzs: [SiZhu]):
    # 年日支查其余支
    targets = [nian_zhi, ri_zhi]
    for target in targets:
        for dz in dzs:
            if dz == target:
                continue
            if dz.text == "无":
                continue
            key = target.text + dz.text
            if key in ["戌亥", "亥戌"]:
                dz.shenshas.append("天罗")
            elif key in ["辰巳", "巳辰"]:
                dz.shenshas.append("地网")


def __check_tao_hua__(nian_zhi: SiZhu, ri_zhi: SiZhu, dzs: [SiZhu]):
    # 年日支查其余支
    key = "桃花"
    targets = [nian_zhi, ri_zhi]
    for target in targets:
        for dz in dzs:
            if dz == target:
                continue
            if dz.text == "无":
                continue
            if target.text in ["申", "子", "辰"] and dz.text == "酉":
                dz.shenshas.append(key)
            elif target.text in ["寅", "午", "戌"] and dz.text == "卯":
                dz.shenshas.append(key)
            elif target.text in ["巳", "酉", "丑"] and dz.text == "午":
                dz.shenshas.append(key)
            elif target.text in ["亥", "卯", "未"] and dz.text == "子":
                dz.shenshas.append(key)
