import math
from datetime import datetime

from wx.dbTools import DBWanNianLi, DBTaiYangShi, DBAreaInfo
from yixue import Configs
from yixue.Models import SiZhuResult


def __calc_time_delta__(diff_hour: int, diff_minute: int, diff_second: int):
    import datetime
    return datetime.timedelta(hours=diff_hour, minutes=diff_minute, seconds=diff_second)


def __get_tiangan_start_of_hour__(ri_gan: str):
    if ri_gan == "甲" or ri_gan == "己":
        return "甲"
    if ri_gan == "乙" or ri_gan == "庚":
        return "丙"
    if ri_gan == "丙" or ri_gan == "辛":
        return "戊"
    if ri_gan == "丁" or ri_gan == "壬":
        return "庚"
    if ri_gan == "戊" or ri_gan == "癸":
        return "壬"


def __get_next_by_dist__(ary: [str], item: str, dist: int):
    result = None
    if ary.__contains__(item):
        index = ary.index(item)
        index += dist
        ln = len(ary)
        while index >= ln:
            index -= ln
        result = ary[index]
    return result


def __next_tiangan__(tiangan: str, dist: int):
    return __get_next_by_dist__(Configs.config_tiangan_list, tiangan, dist)


def __next_dizhi__(dizhi: str, dist: int):
    return __get_next_by_dist__(Configs.config_dizhi_list, dizhi, dist)


def __get_distance_of_hour__(hour: int):
    """
    根据时间换算成当前时间所在的天干地支距离初始天干地支的距离
    """
    dist = 0
    if hour == 23 or hour == 0:
        dist = 0
    elif hour == 1 or hour == 2:
        dist = 1
    elif hour == 3 or hour == 4:
        dist = 2
    elif hour == 5 or hour == 6:
        dist = 3
    elif hour == 7 or hour == 8:
        dist = 4
    elif hour == 9 or hour == 10:
        dist = 5
    elif hour == 11 or hour == 12:
        dist = 6
    elif hour == 13 or hour == 14:
        dist = 7
    elif hour == 15 or hour == 16:
        dist = 8
    elif hour == 17 or hour == 18:
        dist = 9
    elif hour == 19 or hour == 20:
        dist = 10
    elif hour == 21 or hour == 22:
        dist = 11
    return dist


def __get_dizhi_by_hour__(hour: int):
    """
    地支只根据时辰计算，是固定的
    """
    index = 0
    if hour == 23 or hour == 0:
        index = 0
    elif hour == 1 or hour == 2:
        index = 1
    elif hour == 3 or hour == 4:
        index = 2
    elif hour == 5 or hour == 6:
        index = 3
    elif hour == 7 or hour == 8:
        index = 4
    elif hour == 9 or hour == 10:
        index = 5
    elif hour == 11 or hour == 12:
        index = 6
    elif hour == 13 or hour == 14:
        index = 7
    elif hour == 15 or hour == 16:
        index = 8
    elif hour == 17 or hour == 18:
        index = 9
    elif hour == 19 or hour == 20:
        index = 10
    elif hour == 21 or hour == 22:
        index = 11
    return Configs.config_dizhi_list[index]


def run(year: int, month: int, day: int, know_time: bool, hour: int, minute: int, sheng: str, shi: str,
        qu: str) -> SiZhuResult:
    """
        返回数组，顺序是：[年干、年支、月干、月支、日干、日支、时干、时支]
    """
    tm1 = datetime.now()
    wnl = DBWanNianLi.query_first_by(nian=year, yue=month, ri=day)
    tm2 = datetime.now()
    print("    query wan nian li use:", tm2 - tm1)
    if wnl is None:
        return None

    result = SiZhuResult()
    result.sheng = sheng
    result.shi = shi
    result.qu = qu
    result.nian_gan = wnl.nian_gan
    result.nian_zhi = wnl.nian_zhi
    result.yue_gan = wnl.yue_gan
    result.yue_zhi = wnl.yue_zhi
    result.ri_gan = wnl.ri_gan
    result.ri_zhi = wnl.ri_zhi

    if not know_time:
        result.shi_gan = "无"
        result.shi_zhi = "无"
        hour = 0
        minute = 0

    dtm = datetime(year=year, month=month, day=day, hour=hour, minute=minute)
    tys = DBTaiYangShi.query_first_by(month=month, day=day)

    if shi == "-" or qu == "-":
        sheng = "未知地区"  # 防止客户端过来的数据有错
    area = DBAreaInfo.query_first_by(sheng=sheng, shi=shi, qu=qu)
    if area is None:
        sheng = "未知地区"
        shi = "-"
        qu = "-"
        area = DBAreaInfo.query_first_by(sheng=sheng, shi=shi, qu=qu)
    if wnl is not None and tys is not None and area is not None:
        # 计算太阳时导致的偏移，正数减，负数加
        diff_minute = math.trunc(tys.diff / 60)
        diff_second = tys.diff - diff_minute * 60
        tys_delta = __calc_time_delta__(0, diff_minute, diff_second)

        # 计算地区导致的时间偏移，23点减，0点加
        area_delta_hour = area.hour
        area_delta_minute = area.minute
        area_delta_second = area.second
        if area.hour == 23:
            area_delta_hour = 0
            if area.second != 0:
                area_delta_second = -(60 - area.second)
                area_delta_minute = -(59 - area.minute)
            else:
                area_delta_second = 0
                area_delta_minute = -(60 - area.minute)
        area_delta = __calc_time_delta__(area_delta_hour, area_delta_minute, area_delta_second)

        user_date = dtm + area_delta - tys_delta
        result.tai_yang_shi = user_date
        # print("真太阳时：", user_date)

        # 用真太阳时计算时柱
        if know_time is True:
            start_tiangan = __get_tiangan_start_of_hour__(wnl.ri_gan)
            dist = __get_distance_of_hour__(user_date.hour)
            shi_gan = __next_tiangan__(start_tiangan, dist)
            shi_zhi = __get_dizhi_by_hour__(user_date.hour)
            result.shi_gan = shi_gan
            result.shi_zhi = shi_zhi
    return result
