from yixue import Configs
from yixue.Models import BaZi, TaoHuaZhu, LiuNian, LiuYue, DaYun


def calc_yinyuan(bazi: BaZi):
    taphuazhus = __find_taohua_zhu__(bazi)
    if len(taphuazhus) == 0:
        __fix_yinyuan_by_chong__(bazi)
    else:
        __fix_yinyuan__(bazi, taphuazhus)


def __find_taohua_zhu__(bazi: BaZi):
    # 桃花柱的等级：以男命为例
    # 5、财星。4、官星。3、藏干带财官。2、大运带财。1、大运带官。0、带印的柱(只在原局找)。
    result: [TaoHuaZhu] = []
    if bazi.is_man:
        if bazi.ri_zhi.shi_shen in ["财", "才"]:
            pass
        else:
            __find_taohuazhu_in_bazi__(bazi, ["财", "才"], [], 5, result, False)
            __find_taohuazhu_in_bazi__(bazi, ["官", "杀"], ["财", "才"], 4, result, False)
            __find_taohuazhu_in_bazi__(bazi, ["财", "才", "官", "杀"], ["财", "才", "官", "杀"], 3, result, True)
            __find_taohuazhu_in_dayun__(bazi, ["财", "才"], [], 2, result)
            __find_taohuazhu_in_dayun__(bazi, ["官", "杀"], ["财", "才"], 1, result)
            # __find_taohuazhu_in_bazi__(bazi, ["印", "枭"], ["财", "才", "官", "杀"], 1, result, False)
            # for thz in result:
            #     print(thz.level, thz.gan_zhu.wei_zhi, thz.gan_zhu.text, thz.zhi_zhu.wei_zhi, thz.zhi_zhu.text)
    else:
        if bazi.ri_zhi.shi_shen in ["官", "杀"]:
            pass
        else:
            __find_taohuazhu_in_bazi__(bazi, ["官", "杀"], [], 5, result, False)
            __find_taohuazhu_in_bazi__(bazi, ["财", "才"], ["官", "杀"], 4, result, False)
            __find_taohuazhu_in_bazi__(bazi, ["财", "才", "官", "杀"], ["财", "才", "官", "杀"], 3, result, True)
            __find_taohuazhu_in_dayun__(bazi, ["官", "杀"], [], 2, result)
            __find_taohuazhu_in_dayun__(bazi, ["财", "才"], ["官", "杀"], 1, result)

    return result


def __fix_yinyuan__(bazi: BaZi, taohuazhus: [TaoHuaZhu]):
    pei_ou_zhu = TaoHuaZhu(bazi.ri_gan, bazi.ri_zhi, 0)
    pei_ou_zhu.tg_keys = __calc_gaoneng_keys__(pei_ou_zhu.gan_zhu.text, ["合"])
    pei_ou_zhu.dz_keys = __calc_gaoneng_keys__(pei_ou_zhu.zhi_zhu.text, ["合", "半合"])
    for thz in taohuazhus:
        thz.tg_keys = __calc_gaoneng_keys__(thz.gan_zhu.text, ["合"])
        thz.dz_keys = __calc_gaoneng_keys__(thz.zhi_zhu.text, ["合", "半合"])

        for dayun in bazi.dayun:
            if not dayun.is_dayun:
                continue
            if dayun.start_age >= 60 or dayun.end_age <= 10:
                continue
            if thz.level <= 2:  # 从大运找的桃花柱，只在当前大运有效
                if thz.gan_zhu != dayun.gan_zhu and thz.zhi_zhu != dayun.zhi_zhu:
                    continue
            for liunian in dayun.liunian:
                if liunian.age <= 10 or liunian.age >= 60:
                    continue
                is_yin_yuan_nian = __do_fix__(bazi, pei_ou_zhu, thz, dayun, liunian, None)
                if is_yin_yuan_nian:
                    for liuyue in liunian.liuyue:
                        __do_fix__(bazi, pei_ou_zhu, thz, dayun, liunian, liuyue)


def __do_fix__(bazi: BaZi, main_zhu: TaoHuaZhu, taohua: TaoHuaZhu, dayun: DaYun, liunian: LiuNian, liuyue: LiuYue):
    tgs = [bazi.nian_gan.text, bazi.yue_gan.text, bazi.ri_gan.text, bazi.shi_gan.text, dayun.gan_text,
           liunian.gan_text]
    dzs = [bazi.nian_zhi.text, bazi.yue_zhi.text, bazi.ri_zhi.text, bazi.shi_zhi.text, dayun.zhi_text,
           liunian.zhi_text]
    if liuyue is not None:
        tgs.append(liuyue.gan_text)
        dzs.append(liuyue.zhi_text)
    target_index = len(tgs) - 1
    tip = ""

    fixed = False
    if tgs[target_index] in main_zhu.tg_keys:  # 适配日柱天干
        index = __index_of_keys__(taohua.dz_keys, dzs)
        if index != -1:  # 适配桃花地支
            fixed = True
            tip = main_zhu.gan_zhu.text + tgs[target_index] + "、" + taohua.zhi_zhu.text + dzs[index]
    elif dzs[target_index] in main_zhu.dz_keys:  # 适配日柱地支
        index = __index_of_keys__(taohua.tg_keys, tgs)
        if index != -1:  # 适配桃花天干
            fixed = True
            tip = main_zhu.zhi_zhu.text + dzs[target_index] + "、" + taohua.gan_zhu.text + tgs[index]

    if fixed:
        if liuyue is None:
            if liunian.yinyuan_level < taohua.level:
                liunian.yinyuan_level = taohua.level
                liunian.yinyuan_tip = tip
        else:
            if liuyue.yinyuan_level < taohua.level:
                liuyue.yinyuan_level = taohua.level
                liuyue.yinyuan_tip = tip
    return fixed


def __fix_yinyuan_by_chong__(bazi: BaZi):
    chongs = __calc_gaoneng_keys__(bazi.ri_zhi.text, ["冲"])
    for dayun in bazi.dayun:
        if not dayun.is_dayun:
            continue
        if dayun.start_age >= 60 or dayun.end_age <= 10:
            continue
        for liunian in dayun.liunian:
            if liunian.age <= 10 or liunian.age >= 60:
                continue
            if liunian.zhi_text in chongs:
                liunian.yinyuan_level = 5
                liunian.yinyuan_tip = bazi.ri_zhi.text + liunian.zhi_text + "冲"
                for liuyue in liunian.liuyue:
                    if liuyue.zhi_text in chongs:
                        liuyue.yinyuan_level = 1
                        liuyue.yinyuan_tip = bazi.ri_zhi.text + liuyue.zhi_text + "冲"


def __index_of_keys__(keys: [str], ary: [str]):
    index = -1
    for key in keys:
        if key in ary:
            index = ary.index(key)
        break
    return index


def __calc_gaoneng_keys__(text: str, guanxis: [str]):
    result = []
    gns = Configs.get_value(Configs.config_gaoneng_key, text)
    if gns is not None:
        for gn in gns:
            if gn["guanxi"] in guanxis:
                other = str(gn["name"]).replace(gn["guanxi"], "").replace(text, "")
                result.append(other)
    return result


def __find_taohuazhu_in_dayun__(bazi: BaZi, includes: [str], excludes: [str], level: int, result: [TaoHuaZhu]):
    for dayun in bazi.dayun:
        if not dayun.is_dayun:
            continue
        if dayun.gan_zhu.shi_shen in excludes or dayun.zhi_zhu.shi_shen in excludes:
            continue
        if dayun.gan_zhu.shi_shen in includes or dayun.zhi_zhu.shi_shen in includes:
            result.append(TaoHuaZhu(dayun.gan_zhu, dayun.zhi_zhu, level))


def __find_taohuazhu_in_bazi__(bazi: BaZi, includes: [str], excludes: [str], level: int, result: [TaoHuaZhu],
                               search_canggan: bool):
    for index, gan in enumerate(bazi.all_tian_gans()):
        if gan == bazi.ri_gan:
            continue
        zhi = bazi.all_di_zhis()[index]
        if search_canggan is False:
            if gan.shi_shen in excludes or zhi.shi_shen in excludes:
                continue
            if gan.shi_shen in includes or zhi.shi_shen in includes:
                result.append(TaoHuaZhu(gan, zhi, level))
        else:
            if gan.shi_shen in excludes or zhi.shi_shen in excludes:
                continue
            if gan.gan1_shi_shen is not None and gan.gan1_shi_shen in includes:
                result.append(TaoHuaZhu(gan, zhi, level))
            elif gan.gan2_shi_shen is not None and gan.gan2_shi_shen in includes:
                result.append(TaoHuaZhu(gan, zhi, level))
            elif gan.gan3_shi_shen is not None and gan.gan3_shi_shen in excludes:
                result.append(TaoHuaZhu(gan, zhi, level))
            elif zhi.gan1_shi_shen is not None and zhi.gan1_shi_shen in includes:
                result.append(TaoHuaZhu(gan, zhi, level))
            elif zhi.gan2_shi_shen is not None and zhi.gan2_shi_shen in includes:
                result.append(TaoHuaZhu(gan, zhi, level))
            elif zhi.gan3_shi_shen is not None and zhi.gan3_shi_shen == excludes:
                result.append(TaoHuaZhu(gan, zhi, level))
