from yixue import Configs
from yixue.Models import BaZi, GaoNeng, ZhenDuanItem, ZhenDuan, SiZhu


# 看高能的逻辑：
# 合，直接得到，通过什么得到了什么
# 暗合，合的基础上，转暗中。
# 冲：通过怎样的变动得到，能否得到，被什么化解（六合、半合、暗合）(保护没写，讲道理冲不一定是坏事，未必要化解)
# 刑：会被什么拖住，产生什么冲突，被什么化解（六合、半合、暗合）(刑冲没写)
# 穿：什么受伤严重，被什么化解（六合、半合、暗合）
# 破：是否得到，没得到忽略，有得到的话得到的什么不完整，被什么化解（六合、半合、暗合）(暂时没写，更前面的重复度高)
# 三会：什么的力量很强，但是要小心什么破局，破局后呈现什么后果，后果被什么化解（六合、半合、暗合）(暂时没写)


def run(bazi: BaZi):
    resolved_keys: [str] = []  # 避免14柱和41柱都被解读一次
    __about_he__(bazi, resolved_keys)
    __about_an_he__(bazi, resolved_keys)
    __about_chong__(bazi, resolved_keys)
    __about_xing__(bazi, resolved_keys)
    __about_chuan__(bazi, resolved_keys)


def __about_he__(bazi: BaZi, resolved_keys: [str]):
    for sz in bazi.all_zhus():
        ct_he = 0
        more_shishen = ""
        he_key = ""
        for gn in sz.gaonengs:
            if gn.guanxi == "合":
                __do_check__(gn, bazi, resolved_keys, Configs.config_gaoneng_he)
                ct_he += 1
                more_shishen = gn.shi_shen2
                he_key = __get_map_key__(gn)
        if ct_he > 1:
            __check_zheng_he__(sz.shi_shen, more_shishen, he_key, bazi.zhenduan.gaoneng)


def __about_an_he__(bazi: BaZi, resolved_keys: [str]):
    for sz in bazi.all_zhus():
        for gn in sz.gaonengs:
            if gn.guanxi == "暗合":
                __do_check__(gn, bazi, resolved_keys, Configs.config_gaoneng_anhe)


def __about_chong__(bazi: BaZi, resolved_keys: [str]):
    for sz in bazi.all_zhus():
        for gn in sz.gaonengs:
            if gn.guanxi == "冲":
                __do_check__(gn, bazi, resolved_keys, Configs.config_gaoneng_chong)


def __about_xing__(bazi: BaZi, resolved_keys: [str]):
    for sz in bazi.all_zhus():
        for gn in sz.gaonengs:
            if gn.guanxi == "刑":
                __do_check_xing__(gn, bazi, resolved_keys)


def __about_chuan__(bazi: BaZi, resolved_keys: [str]):
    for sz in bazi.all_zhus():
        for gn in sz.gaonengs:
            if gn.guanxi == "穿":
                __do_check__(gn, bazi, resolved_keys, Configs.config_gaoneng_chuan)


def __do_check__(gn: GaoNeng, bazi: BaZi, resolved_keys: [str], data_src: {}):
    resolved_key = __get_gongneng_key(gn)
    if resolved_keys.__contains__(resolved_key):
        return

    data = Configs.get_value(data_src, gn.shi_shen1 + gn.shi_shen2)
    if data is None:
        data = Configs.get_value(data_src, gn.shi_shen2 + gn.shi_shen1)
    if data is not None:
        for dt in data:

            jiedu = dt["解读"]
            if gn.guanxi == "冲":
                power1 = __get_power__(gn.shi_shen1, bazi)
                power2 = __get_power__(gn.shi_shen2, bazi)
                tag = "平"
                if power1 > power2 * 2:
                    tag = "胜"
                elif power1 * 2 < power2:
                    tag = "负"
                jiedu += dt[tag]

            full_name1 = __get_shishen_full_name__(gn.shi_shen1)
            full_name2 = __get_shishen_full_name__(gn.shi_shen2)
            content = full_name1 + gn.guanxi + full_name2 + "，" + jiedu

            if gn.guanxi in ["穿"]:
                zhu1 = __get_zhu_by_weizhi__(gn.wei_zhi1, bazi)
                other_text = __get_baohuzhe_in_zhu__(zhu1)
                if other_text is None:
                    zhu2 = __get_zhu_by_weizhi__(gn.wei_zhi2, bazi)
                    other_text = __get_baohuzhe_in_zhu__(zhu2)
                if other_text is not None:
                    other_wuxing = Configs.get_value(Configs.config_wuxing, other_text)
                    content += "(受" + other_text + other_wuxing + "的保护，负面的影响会被削弱)"

            item = ZhenDuanItem(0, content)
            item.key = gn.text

            if dt["类别"] == "建议":
                item.set_selected(True)
            __add_gaoneng_zhenduan__(__get_map_key__(gn), item, bazi.zhenduan.gaoneng)
    else:
        print("没找到匹配的关系在：", gn.shi_shen1 + gn.shi_shen2, gn.guanxi)
    resolved_keys.append(resolved_key)


def __do_check_xing__(gn: GaoNeng, bazi: BaZi, resolved_keys: [str]):
    resolved_key = __get_gongneng_key(gn)
    if resolved_keys.__contains__(resolved_key):
        return
    data = Configs.get_value(Configs.config_gaoneng_xing, gn.text)
    if data is not None:
        content = ""
        jie_fang = ""
        for chu in data["刑出"]:
            chu_shishen = Configs.get_value2(Configs.config_shishen, bazi.ri_gan.text, chu)
            full_name = __get_shishen_full_name__(chu_shishen)
            jie_fang += full_name
            if chu_shishen in ["官", "杀"]:
                content += full_name + "被刑，容易被目标吊着，总是差一点能够到。"
            elif chu_shishen in ["印", "枭"]:
                content += full_name + "被刑，得到的帮助都有代价，甚至与自身深度绑定。"
            elif chu_shishen in ["财", "才"]:
                content += full_name + "被刑，需要持续的投入财富、精力，持续的输血。"
            elif chu_shishen in ["食", "伤"]:
                content += full_name + "被刑，有力没处使，甚至感觉怀才不遇，情绪深陷其中。"
            elif chu_shishen in ["比", "劫"]:
                content += full_name + "被刑，自身、朋友、团队都被深度绑定，陷入漩涡。"

        # chong = data["刑冲"][0]
        # chong_shishen1 = Configs.get_value2(Configs.config_shishen, bazi.ri_gan.text, chong[0])
        # chong_shishen2 = Configs.get_value2(Configs.config_shishen, bazi.ri_gan.text, chong[1])
        # chong_data = Configs.get_value(Configs.config_gaoneng_chong, chong_shishen1 + chong_shishen2)
        # for cd in chong_data:
        #     jiedu = cd["解读"]
        #     power1 = __get_power__(chong_shishen1, bazi)
        #     power2 = __get_power__(chong_shishen2, bazi)
        #     tag = "平"
        #     if power1 > power2 * 2:
        #         tag = "胜"
        #     elif power1 * 2 < power2:
        #         tag = "负"
        #     jiedu += cd[tag]
        #     content += jiedu

        zhu1 = __get_zhu_by_weizhi__(gn.wei_zhi1, bazi)
        other_text = __get_baohuzhe_in_zhu__(zhu1)
        if other_text is None:
            zhu2 = __get_zhu_by_weizhi__(gn.wei_zhi2, bazi)
            other_text = __get_baohuzhe_in_zhu__(zhu2)
        if other_text is not None:
            other_wuxing = Configs.get_value(Configs.config_wuxing, other_text)
            content += "(受" + other_text + other_wuxing + "的保护，负面影响会被削弱)"

        item = ZhenDuanItem(0, content)
        item.key = gn.text
        item.set_selected(True)
        __add_gaoneng_zhenduan__(__get_map_key__(gn), item, bazi.zhenduan.gaoneng)

    else:
        print("没找到匹配的关系在：", gn.text)
    resolved_keys.append(resolved_key)


def __get_baohuzhe_in_zhu__(zhu: SiZhu):
    result = None
    self_text = zhu.text
    for gn in zhu.gaonengs:
        if gn.guanxi == "合":
            result = str(gn.text).replace("合", "").replace(self_text, "")
            break
        elif gn.guanxi == "暗合":
            result = str(gn.text).replace("暗合", "").replace(self_text, "")
            break
    return result


def __get_zhu_by_weizhi__(weizhi: int, bazi: BaZi):
    if weizhi == Configs.const_nian_gan:
        return bazi.nian_gan
    if weizhi == Configs.const_yue_gan:
        return bazi.yue_gan
    if weizhi == Configs.const_ri_gan:
        return bazi.ri_gan
    if weizhi == Configs.const_shi_gan:
        return bazi.shi_gan
    if weizhi == Configs.const_nian_zhi:
        return bazi.nian_zhi
    if weizhi == Configs.const_yue_zhi:
        return bazi.yue_zhi
    if weizhi == Configs.const_ri_zhi:
        return bazi.ri_zhi
    if weizhi == Configs.const_shi_zhi:
        return bazi.shi_zhi


def __check_zheng_he__(single: str, more: str, he_key: str, map: {}):
    tip = Configs.get_value2(Configs.config_zheng_he, single, more)
    if tip is not None:
        item = ZhenDuanItem(0, tip)
        item.set_selected(True)
        __add_gaoneng_zhenduan__(he_key, item, map)
    else:
        print("争合：", single, more, "找不到对一个的描述")


def __add_gaoneng_zhenduan__(map_key: str, item: ZhenDuanItem, map: {}):
    if not map.__contains__(map_key):
        map[map_key] = ZhenDuan()

    if item.defaultSelected:
        found = False
        for obj in map[map_key].jianyi:
            if obj.key == item.key:
                found = True
                break
        if not found:
            map[map_key].jianyi.append(item)
        else:
            item.set_selected(False)
            item.tip = "重复项"
            map[map_key].cankao.append(item)
    else:
        map[map_key].cankao.append(item)


def __get_shishen_full_name__(ss: str):
    if ss == "官":
        return "正官"
    elif ss == "杀":
        return "七杀"
    elif ss == "财":
        return "正财"
    elif ss == "才":
        return "偏财"
    elif ss == "印":
        return "正印"
    elif ss == "枭":
        return "偏印"
    elif ss == "食":
        return "食神"
    elif ss == "伤":
        return "伤官"
    elif ss == "比":
        return "比肩"
    elif ss == "劫":
        return "劫财"
    return None


def __get_power__(shishen: str, bazi: BaZi):
    step = bazi.power_steps[shishen]
    amount = bazi.counter.power_raw[shishen]
    return round(step * amount, 0)


def __get_gongneng_key(gn: GaoNeng):
    key = gn.text + "_"
    indexs = [gn.wei_zhi1, gn.wei_zhi2]
    if gn.arg3 is not None:
        indexs.append(gn.wei_zhi3)
    indexs.sort()
    for i in indexs:
        key += str(i)
    return key


def __is_guanxi__(gn: GaoNeng, shishen1: str, shishen2: str, guanxi: str):
    result = False
    keys = [gn.shi_shen1, gn.shi_shen2]
    if gn.arg3 is not None:
        keys.append(gn.shi_shen3)
    if shishen1 in keys and shishen2 in keys and gn.guanxi == guanxi:
        result = True
    return result


def __get_map_key__(gn: GaoNeng):
    key = gn.text[0] + gn.text[1]
    if gn.arg3 is not None:
        key += gn.text[2]
    return key


def __test_gaoneng__():
    temp = {}
    for tg in Configs.config_tiangan_list:
        for key in Configs.config_gaoneng:
            data = Configs.config_gaoneng[key]
            guanxi = data["guanxi"]
            args = key.replace(guanxi, "")
            ss1 = Configs.get_value2(Configs.config_shishen, tg, args[0])
            ss2 = Configs.get_value2(Configs.config_shishen, tg, args[1])
            k = ss1 + ss2 + guanxi
            if not temp.__contains__(guanxi):
                temp[guanxi] = {}
            if not temp[guanxi].__contains__(ss1):
                temp[guanxi][ss1] = []
            if not temp[guanxi][ss1].__contains__(k):
                temp[guanxi][ss1].append(k)
            if guanxi == "合" and ss1 == "官" and ss2 == "杀":
                print(tg, data)
    print(temp)
